import unittest

from podman import PodmanClient, tests
from podman.domain.secrets import SecretsManager


class SecretsTestCase(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()

        self.client = PodmanClient(base_url=tests.BASE_SOCK)

    def tearDown(self) -> None:
        super().tearDown()

        self.client.close()

    def test_podmanclient(self):
        manager = self.client.secrets
        self.assertIsInstance(manager, SecretsManager)


if __name__ == '__main__':
    unittest.main()



"""PodmanPy Tests."""

# Do not auto-update these from version.py,
#   as test code should be changed to reflect changes in Podman API versions
BASE_SOCK = "unix:///run/api.sock"
LIBPOD_URL = "http://%2Frun%2Fapi.sock/v5.7.0/libpod"
COMPATIBLE_URL = "http://%2Frun%2Fapi.sock/v1.40"



import os
import unittest
import tempfile
from unittest import mock

from podman import api


class PathUtilsTestCase(unittest.TestCase):
    def setUp(self):
        self.xdg_runtime_dir = os.getenv('XDG_RUNTIME_DIR')

    @mock.patch.dict(os.environ, clear=True)
    def test_get_runtime_dir_env_var_set(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            os.environ['XDG_RUNTIME_DIR'] = str(tmpdir)
            self.assertEqual(str(tmpdir), api.path_utils.get_runtime_dir())

    @mock.patch.dict(os.environ, clear=True)
    def test_get_runtime_dir_env_var_not_set(self):
        if not self.xdg_runtime_dir:
            self.skipTest('XDG_RUNTIME_DIR must be set for this test.')
        if self.xdg_runtime_dir.startswith('/run/user/'):
            self.skipTest("XDG_RUNTIME_DIR in /run/user/, can't check")
        self.assertNotEqual(self.xdg_runtime_dir, api.path_utils.get_runtime_dir())

    @mock.patch('os.path.isdir', lambda d: False)
    @mock.patch.dict(os.environ, clear=True)
    def test_get_runtime_dir_env_var_not_set_and_no_run(self):
        """Fake that XDG_RUNTIME_DIR is not set and /run/user/ does not exist."""
        if not self.xdg_runtime_dir:
            self.skipTest('XDG_RUNTIME_DIR must be set to fetch a working dir.')
        self.assertNotEqual(self.xdg_runtime_dir, api.path_utils.get_runtime_dir())


if __name__ == '__main__':
    unittest.main()



import json
import unittest

try:
    # Python >= 3.10
    from collections.abc import Iterator
except ImportError:
    # Python < 3.10
    from collections.abc import Iterator

from unittest.mock import DEFAULT, MagicMock, patch

import requests_mock

from podman import PodmanClient, tests
from podman.domain.containers import Container
from podman.domain.containers_create import CreateMixin
from podman.domain.containers_manager import ContainersManager
from podman.errors import ImageNotFound, NotFound

FIRST_CONTAINER = {
    "Id": "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
    "Image": "quay.io/fedora:latest",
    "Name": "evil_ptolemy",
    "HostConfig": {"LogConfig": {"Type": "json-file"}},
}
SECOND_CONTAINER = {
    "Id": "6dc84cc0a46747da94e4c1571efcc01a756b4017261440b4b8985d37203c3c03",
    "Image": "quay.io/fedora:rc",
    "Name": "good_galileo",
}


class ContainersManagerTestCase(unittest.TestCase):
    """Test ContainersManager area of concern.

    Note:
        Mock responses need to be coded for libpod returns.  The python bindings are responsible
            for mapping to compatible output.
    """

    def setUp(self) -> None:
        super().setUp()

        self.client = PodmanClient(base_url=tests.BASE_SOCK)

    def tearDown(self) -> None:
        super().tearDown()

        self.client.close()

    def test_podmanclient(self):
        manager = self.client.containers
        self.assertIsInstance(manager, ContainersManager)

    @requests_mock.Mocker()
    def test_get(self, mock):
        mock.get(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/json",
            json=FIRST_CONTAINER,
        )

        actual = self.client.containers.get(
            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd"
        )
        self.assertEqual(
            actual.id,
            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
        )

    @requests_mock.Mocker()
    def test_get_404(self, mock):
        mock.get(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/json",
            json={
                "cause": "Container not found.",
                "message": "Container not found.",
                "response": 404,
            },
            status_code=404,
        )

        with self.assertRaises(NotFound):
            self.client.containers.get(
                "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd"
            )

    @requests_mock.Mocker()
    def test_list_empty(self, mock):
        mock.get(
            tests.LIBPOD_URL + "/containers/json",
            text="[]",
        )
        actual = self.client.containers.list()
        self.assertListEqual(actual, [])

    @requests_mock.Mocker()
    def test_list(self, mock):
        mock.get(
            tests.LIBPOD_URL + "/containers/json",
            json=[FIRST_CONTAINER, SECOND_CONTAINER],
        )
        actual = self.client.containers.list()
        self.assertIsInstance(actual, list)

        self.assertEqual(
            actual[0].id,
            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
        )
        self.assertEqual(
            actual[1].id,
            "6dc84cc0a46747da94e4c1571efcc01a756b4017261440b4b8985d37203c3c03",
        )

    @requests_mock.Mocker()
    def test_list_filtered(self, mock):
        mock.get(
            tests.LIBPOD_URL + "/containers/json?"
            "all=True"
            "&filters=%7B"
            "%22before%22%3A"
            "+%5B%226dc84cc0a46747da94e4c1571efcc01a756b4017261440b4b8985d37203c3c03%22%5D%2C"
            "+%22since%22%3A"
            "+%5B%2287e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd%22%5D%2C"
            "+%22status%22%3A+%5B%22running%22%5D%7D",
            json=[FIRST_CONTAINER, SECOND_CONTAINER],
        )
        actual = self.client.containers.list(
            all=True,
            filters={"status": "running"},
            since="87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
            before="6dc84cc0a46747da94e4c1571efcc01a756b4017261440b4b8985d37203c3c03",
        )
        self.assertIsInstance(actual, list)

        self.assertEqual(
            actual[0].id,
            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
        )
        self.assertEqual(
            actual[1].id,
            "6dc84cc0a46747da94e4c1571efcc01a756b4017261440b4b8985d37203c3c03",
        )

    @requests_mock.Mocker()
    def test_list_no_filters(self, mock):
        mock.get(
            tests.LIBPOD_URL + "/containers/json",
            json=[FIRST_CONTAINER, SECOND_CONTAINER],
        )
        actual = self.client.containers.list()
        self.assertIsInstance(actual, list)

        self.assertEqual(
            actual[0].id,
            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
        )
        self.assertEqual(
            actual[1].id,
            "6dc84cc0a46747da94e4c1571efcc01a756b4017261440b4b8985d37203c3c03",
        )

    @requests_mock.Mocker()
    def test_list_sparse_libpod_default(self, mock):
        mock.get(
            tests.LIBPOD_URL + "/containers/json",
            json=[FIRST_CONTAINER, SECOND_CONTAINER],
        )
        actual = self.client.containers.list()
        self.assertIsInstance(actual, list)

        self.assertEqual(
            actual[0].id, "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd"
        )
        self.assertEqual(
            actual[1].id, "6dc84cc0a46747da94e4c1571efcc01a756b4017261440b4b8985d37203c3c03"
        )

        # Verify that no individual reload() calls were made for sparse=True (default)
        # Should be only 1 request for the list endpoint
        self.assertEqual(len(mock.request_history), 1)
        # lower() needs to be enforced since the mocked url is transformed as lowercase and
        # this avoids %2f != %2F errors. Same applies for other instances of assertEqual
        self.assertEqual(mock.request_history[0].url, tests.LIBPOD_URL.lower() + "/containers/json")

    @requests_mock.Mocker()
    def test_list_sparse_libpod_false(self, mock):
        mock.get(
            tests.LIBPOD_URL + "/containers/json",
            json=[FIRST_CONTAINER, SECOND_CONTAINER],
        )
        # Mock individual container detail endpoints for reload() calls
        # that are done for sparse=False
        mock.get(
            tests.LIBPOD_URL + f"/containers/{FIRST_CONTAINER['Id']}/json",
            json=FIRST_CONTAINER,
        )
        mock.get(
            tests.LIBPOD_URL + f"/containers/{SECOND_CONTAINER['Id']}/json",
            json=SECOND_CONTAINER,
        )
        actual = self.client.containers.list(sparse=False)
        self.assertIsInstance(actual, list)

        self.assertEqual(
            actual[0].id, "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd"
        )
        self.assertEqual(
            actual[1].id, "6dc84cc0a46747da94e4c1571efcc01a756b4017261440b4b8985d37203c3c03"
        )

        # Verify that individual reload() calls were made for sparse=False
        # Should be 3 requests total: 1 for list + 2 for individual container details
        self.assertEqual(len(mock.request_history), 3)

        # Verify the list endpoint was called first
        self.assertEqual(mock.request_history[0].url, tests.LIBPOD_URL.lower() + "/containers/json")

        # Verify the individual container detail endpoints were called
        individual_urls = {req.url for req in mock.request_history[1:]}
        expected_urls = {
            tests.LIBPOD_URL.lower() + f"/containers/{FIRST_CONTAINER['Id']}/json",
            tests.LIBPOD_URL.lower() + f"/containers/{SECOND_CONTAINER['Id']}/json",
        }
        self.assertEqual(individual_urls, expected_urls)

    @requests_mock.Mocker()
    def test_list_sparse_compat_default(self, mock):
        mock.get(
            tests.COMPATIBLE_URL + "/containers/json",
            json=[FIRST_CONTAINER, SECOND_CONTAINER],
        )
        # Mock individual container detail endpoints for reload() calls
        # that are done for sparse=False
        mock.get(
            tests.COMPATIBLE_URL + f"/containers/{FIRST_CONTAINER['Id']}/json",
            json=FIRST_CONTAINER,
        )
        mock.get(
            tests.COMPATIBLE_URL + f"/containers/{SECOND_CONTAINER['Id']}/json",
            json=SECOND_CONTAINER,
        )
        actual = self.client.containers.list(compatible=True)
        self.assertIsInstance(actual, list)

        self.assertEqual(
            actual[0].id, "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd"
        )
        self.assertEqual(
            actual[1].id, "6dc84cc0a46747da94e4c1571efcc01a756b4017261440b4b8985d37203c3c03"
        )

        # Verify that individual reload() calls were made for compat default (sparse=True)
        # Should be 3 requests total: 1 for list + 2 for individual container details
        self.assertEqual(len(mock.request_history), 3)
        self.assertEqual(
            mock.request_history[0].url, tests.COMPATIBLE_URL.lower() + "/containers/json"
        )

        # Verify the individual container detail endpoints were called
        individual_urls = {req.url for req in mock.request_history[1:]}
        expected_urls = {
            tests.COMPATIBLE_URL.lower() + f"/containers/{FIRST_CONTAINER['Id']}/json",
            tests.COMPATIBLE_URL.lower() + f"/containers/{SECOND_CONTAINER['Id']}/json",
        }
        self.assertEqual(individual_urls, expected_urls)

    @requests_mock.Mocker()
    def test_list_sparse_compat_true(self, mock):
        mock.get(
            tests.COMPATIBLE_URL + "/containers/json",
            json=[FIRST_CONTAINER, SECOND_CONTAINER],
        )
        actual = self.client.containers.list(sparse=True, compatible=True)
        self.assertIsInstance(actual, list)

        self.assertEqual(
            actual[0].id, "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd"
        )
        self.assertEqual(
            actual[1].id, "6dc84cc0a46747da94e4c1571efcc01a756b4017261440b4b8985d37203c3c03"
        )

        # Verify that no individual reload() calls were made for sparse=True
        # Should be only 1 request for the list endpoint
        self.assertEqual(len(mock.request_history), 1)
        self.assertEqual(
            mock.request_history[0].url, tests.COMPATIBLE_URL.lower() + "/containers/json"
        )

    @requests_mock.Mocker()
    def test_prune(self, mock):
        mock.post(
            tests.LIBPOD_URL + "/containers/prune",
            json=[
                {
                    "Id": "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
                    "Size": 1024,
                },
                {
                    "Id": "6dc84cc0a46747da94e4c1571efcc01a756b4017261440b4b8985d37203c3c03",
                    "Size": 1024,
                },
            ],
        )
        actual = self.client.containers.prune()
        self.assertDictEqual(
            actual,
            {
                "ContainersDeleted": [
                    "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
                    "6dc84cc0a46747da94e4c1571efcc01a756b4017261440b4b8985d37203c3c03",
                ],
                "SpaceReclaimed": 2048,
            },
        )

    @requests_mock.Mocker()
    def test_create(self, mock):
        mock.post(
            tests.LIBPOD_URL + "/containers/create",
            status_code=201,
            json={
                "Id": "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
                "Warnings": [],
            },
        )
        mock.get(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/json",
            json=FIRST_CONTAINER,
        )

        actual = self.client.containers.create("fedora", "/usr/bin/ls", cpu_count=9999)
        self.assertIsInstance(actual, Container)

    @requests_mock.Mocker()
    def test_create_404(self, mock):
        mock.post(
            tests.LIBPOD_URL + "/containers/create",
            status_code=404,
            json={
                "cause": "Image not found",
                "message": "Image not found",
                "response": 404,
            },
        )
        with self.assertRaises(ImageNotFound):
            self.client.containers.create("fedora", "/usr/bin/ls", cpu_count=9999)

    @requests_mock.Mocker()
    def test_create_parse_host_port(self, mock):
        mock_response = MagicMock()
        mock_response.json = lambda: {
            "Id": "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
            "Size": 1024,
        }
        self.client.containers.client.post = MagicMock(return_value=mock_response)
        mock.get(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/json",
            json=FIRST_CONTAINER,
        )

        port_str = {"2233": 3333}
        port_str_protocol = {"2244/tcp": 3344}
        port_int = {2255: 3355}
        ports = {**port_str, **port_str_protocol, **port_int}
        self.client.containers.create("fedora", "/usr/bin/ls", ports=ports)

        self.client.containers.client.post.assert_called()
        expected_ports = [
            {
                "container_port": 2233,
                "host_port": 3333,
                "protocol": "tcp",
            },
            {
                "container_port": 2244,
                "host_port": 3344,
                "protocol": "tcp",
            },
            {
                "container_port": 2255,
                "host_port": 3355,
                "protocol": "tcp",
            },
        ]
        actual_ports = json.loads(self.client.containers.client.post.call_args[1]["data"])[
            "portmappings"
        ]
        self.assertEqual(expected_ports, actual_ports)

    @requests_mock.Mocker()
    def test_create_userns_mode_simple(self, mock):
        mock_response = MagicMock()
        mock_response.json = lambda: {
            "Id": "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
            "Size": 1024,
        }
        self.client.containers.client.post = MagicMock(return_value=mock_response)
        mock.get(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/json",
            json=FIRST_CONTAINER,
        )

        userns = "keep-id"
        self.client.containers.create("fedora", "/usr/bin/ls", userns_mode=userns)
        self.client.containers.client.post.assert_called()
        expected_userns = {"nsmode": userns}

        actual_userns = json.loads(self.client.containers.client.post.call_args[1]["data"])[
            "userns"
        ]
        self.assertEqual(expected_userns, actual_userns)

    @requests_mock.Mocker()
    def test_create_userns_mode_dict(self, mock):
        mock_response = MagicMock()
        mock_response.json = lambda: {
            "Id": "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
            "Size": 1024,
        }
        self.client.containers.client.post = MagicMock(return_value=mock_response)
        mock.get(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/json",
            json=FIRST_CONTAINER,
        )

        userns = {"nsmode": "keep-id", "value": "uid=900"}
        self.client.containers.create("fedora", "/usr/bin/ls", userns_mode=userns)
        self.client.containers.client.post.assert_called()
        expected_userns = dict(**userns)

        actual_userns = json.loads(self.client.containers.client.post.call_args[1]["data"])[
            "userns"
        ]
        self.assertEqual(expected_userns, actual_userns)

    def test_create_unsupported_key(self):
        with self.assertRaises(TypeError):
            self.client.containers.create("fedora", "/usr/bin/ls", blkio_weight=100.0)

    def test_create_unknown_key(self):
        with self.assertRaises(TypeError):
            self.client.containers.create("fedora", "/usr/bin/ls", unknown_key=100.0)

    @requests_mock.Mocker()
    def test_create_convert_env_list_to_dict(self, mock):
        env_list1 = ["FOO=foo", "BAR=bar"]
        # Test valid list
        converted_dict1 = {"FOO": "foo", "BAR": "bar"}
        self.assertEqual(CreateMixin._convert_env_list_to_dict(env_list1), converted_dict1)

        # Test empty string
        env_list2 = ["FOO=foo", ""]
        self.assertRaises(ValueError, CreateMixin._convert_env_list_to_dict, env_list2)

        # Test non iterable
        env_list3 = ["FOO=foo", None]
        self.assertRaises(TypeError, CreateMixin._convert_env_list_to_dict, env_list3)

        # Test iterable with non string element
        env_list4 = ["FOO=foo", []]
        self.assertRaises(TypeError, CreateMixin._convert_env_list_to_dict, env_list4)

        # Test empty list
        env_list5 = []
        converted_dict5 = {}
        self.assertEqual(CreateMixin._convert_env_list_to_dict(env_list5), converted_dict5)

        # Test single valid environment variable
        env_list6 = ["SINGLE=value"]
        converted_dict6 = {"SINGLE": "value"}
        self.assertEqual(CreateMixin._convert_env_list_to_dict(env_list6), converted_dict6)

        # Test environment variable with empty value
        env_list7 = ["EMPTY="]
        converted_dict7 = {"EMPTY": ""}
        self.assertEqual(CreateMixin._convert_env_list_to_dict(env_list7), converted_dict7)

        # Test environment variable with multiple equals signs
        env_list8 = ["URL=https://example.com/path?param=value"]
        converted_dict8 = {"URL": "https://example.com/path?param=value"}
        self.assertEqual(CreateMixin._convert_env_list_to_dict(env_list8), converted_dict8)

        # Test environment variable with spaces in value
        env_list9 = ["MESSAGE=Hello World", "PATH=/usr/local/bin:/usr/bin"]
        converted_dict9 = {"MESSAGE": "Hello World", "PATH": "/usr/local/bin:/usr/bin"}
        self.assertEqual(CreateMixin._convert_env_list_to_dict(env_list9), converted_dict9)

        # Test environment variable with special characters
        env_list10 = ["SPECIAL=!@#$%^&*()_+-=[]{}|;':\",./<>?"]
        converted_dict10 = {"SPECIAL": "!@#$%^&*()_+-=[]{}|;':\",./<>?"}
        self.assertEqual(CreateMixin._convert_env_list_to_dict(env_list10), converted_dict10)

        # Test environment variable with numeric values
        env_list11 = ["PORT=8080", "TIMEOUT=30"]
        converted_dict11 = {"PORT": "8080", "TIMEOUT": "30"}
        self.assertEqual(CreateMixin._convert_env_list_to_dict(env_list11), converted_dict11)

        # Test environment variable with boolean-like values
        env_list12 = ["DEBUG=true", "VERBOSE=false", "ENABLED=1", "DISABLED=0"]
        converted_dict12 = {
            "DEBUG": "true",
            "VERBOSE": "false",
            "ENABLED": "1",
            "DISABLED": "0",
        }
        self.assertEqual(CreateMixin._convert_env_list_to_dict(env_list12), converted_dict12)

        # Test environment variable with whitespace in key (should preserve)
        env_list13 = [" SPACED_KEY =value", "KEY= spaced_value "]
        converted_dict13 = {" SPACED_KEY ": "value", "KEY": " spaced_value "}
        self.assertEqual(CreateMixin._convert_env_list_to_dict(env_list13), converted_dict13)

        # Test missing equals sign
        env_list14 = ["FOO=foo", "INVALID"]
        self.assertRaises(ValueError, CreateMixin._convert_env_list_to_dict, env_list14)

        # Test environment variable with only equals sign (empty key)
        env_list15 = ["FOO=foo", "=value"]
        self.assertRaises(ValueError, CreateMixin._convert_env_list_to_dict, env_list15)

        # Test environment variable with only whitespace key
        env_list16 = ["FOO=foo", "   =value"]
        self.assertRaises(ValueError, CreateMixin._convert_env_list_to_dict, env_list16)

        # Test whitespace-only string
        env_list17 = ["FOO=foo", "   "]
        self.assertRaises(ValueError, CreateMixin._convert_env_list_to_dict, env_list17)

        # Test various non-string types in list
        env_list18 = ["FOO=foo", 123]
        self.assertRaises(TypeError, CreateMixin._convert_env_list_to_dict, env_list18)

        env_list19 = ["FOO=foo", {"key": "value"}]
        self.assertRaises(TypeError, CreateMixin._convert_env_list_to_dict, env_list19)

        env_list20 = ["FOO=foo", True]
        self.assertRaises(TypeError, CreateMixin._convert_env_list_to_dict, env_list20)

        # Test duplicate keys (last one should win)
        env_list21 = ["KEY=first", "KEY=second", "OTHER=value"]
        converted_dict21 = {"KEY": "second", "OTHER": "value"}
        self.assertEqual(CreateMixin._convert_env_list_to_dict(env_list21), converted_dict21)

        # Test very long environment variable
        long_value = "x" * 1000
        env_list22 = [f"LONG_VAR={long_value}"]
        converted_dict22 = {"LONG_VAR": long_value}
        self.assertEqual(CreateMixin._convert_env_list_to_dict(env_list22), converted_dict22)

        # Test environment variable with newlines and tabs
        env_list23 = ["MULTILINE=line1\nline2\ttabbed"]
        converted_dict23 = {"MULTILINE": "line1\nline2\ttabbed"}
        self.assertEqual(CreateMixin._convert_env_list_to_dict(env_list23), converted_dict23)

        # Test environment variable with unicode characters
        env_list24 = ["UNICODE=こんにちは", "EMOJI=🚀🌟"]
        converted_dict24 = {"UNICODE": "こんにちは", "EMOJI": "🚀🌟"}
        self.assertEqual(CreateMixin._convert_env_list_to_dict(env_list24), converted_dict24)

        # Test case sensitivity
        env_list25 = ["path=/usr/bin", "PATH=/usr/local/bin"]
        converted_dict25 = {"path": "/usr/bin", "PATH": "/usr/local/bin"}
        self.assertEqual(CreateMixin._convert_env_list_to_dict(env_list25), converted_dict25)

    @requests_mock.Mocker()
    def test_run_detached(self, mock):
        mock.post(
            tests.LIBPOD_URL + "/containers/create",
            status_code=201,
            json={
                "Id": "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
                "Warnings": [],
            },
        )
        mock.post(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/start",
            status_code=204,
        )
        mock.get(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/json",
            json=FIRST_CONTAINER,
        )

        with patch.multiple(Container, logs=DEFAULT, wait=DEFAULT, autospec=True) as mock_container:
            mock_container["logs"].return_value = []
            mock_container["wait"].return_value = {"StatusCode": 0}

            actual = self.client.containers.run("fedora", "/usr/bin/ls", detach=True)
            self.assertIsInstance(actual, Container)

    @requests_mock.Mocker()
    def test_run(self, mock):
        mock.post(
            tests.LIBPOD_URL + "/containers/create",
            status_code=201,
            json={
                "Id": "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
                "Warnings": [],
            },
        )
        mock.post(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/start",
            status_code=204,
        )
        mock.get(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/json",
            json=FIRST_CONTAINER,
        )

        mock_logs = (
            b"This is a unittest - line 1",
            b"This is a unittest - line 2",
        )

        with patch.multiple(Container, logs=DEFAULT, wait=DEFAULT, autospec=True) as mock_container:
            mock_container["wait"].return_value = 0

            with self.subTest("Results not streamed"):
                mock_container["logs"].return_value = iter(mock_logs)

                actual = self.client.containers.run("fedora", "/usr/bin/ls")
                self.assertIsInstance(actual, bytes)
                self.assertEqual(actual, b"This is a unittest - line 1This is a unittest - line 2")

            # iter() cannot be reset so subtests used to create new instance
            with self.subTest("Stream results"):
                mock_container["logs"].return_value = iter(mock_logs)

                actual = self.client.containers.run("fedora", "/usr/bin/ls", stream=True)
                self.assertNotIsInstance(actual, bytes)
                self.assertIsInstance(actual, Iterator)
                self.assertEqual(next(actual), b"This is a unittest - line 1")
                self.assertEqual(next(actual), b"This is a unittest - line 2")


if __name__ == "__main__":
    unittest.main()



#   Copyright 2020 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""Base integration test code"""

import logging
import os
import shutil
import uuid

import fixtures

from podman.tests.integration import utils


class IntegrationTest(fixtures.TestWithFixtures):
    """Base Integration test case.

    Notes:
        - Logging for the Podman service configured here for later capture, this configuration is
          inherited by other libraries like unittest and requests.
        - Logging output will start with stdout from the Podman service events, followed by
          results and logging captured by the unittest module test runner.
    """

    podman: str = None

    @classmethod
    def setUpClass(cls) -> None:
        super(fixtures.TestWithFixtures, cls).setUpClass()

        command = os.environ.get("PODMAN_BINARY", "podman")
        if shutil.which(command) is None:
            raise AssertionError(f"'{command}' not found.")
        IntegrationTest.podman = command

        # This log_level is for our python code
        log_level = os.environ.get("PODMAN_LOG_LEVEL", "INFO")
        log_level = logging.getLevelName(log_level)
        logging.basicConfig(level=log_level)

    def setUp(self):
        super().setUp()

        self.log_level = os.environ.get("PODMAN_LOG_LEVEL", "INFO")

        self.test_dir = self.useFixture(fixtures.TempDir()).path
        self.socket_file = os.path.join(self.test_dir, uuid.uuid4().hex)
        self.socket_uri = f'unix://{self.socket_file}'
        self.service_launcher = utils.PodmanLauncher(
            self.socket_uri, podman_path=IntegrationTest.podman, log_level=self.log_level
        )
        self.service_launcher.start()
        self.addCleanup(self.service_launcher.stop)



import json
import pathlib
import unittest
from typing import Any, Optional
from unittest import mock
from unittest.mock import mock_open, patch

from dataclasses import dataclass

from podman import api


class TestUtilsCase(unittest.TestCase):
    def test_format_filters(self) -> None:
        @dataclass
        class TestCase:
            name: str
            input: Any
            expected: Optional[str]

        cases = [
            TestCase(name="empty str", input="", expected=None),
            TestCase(name="str", input="reference=fedora", expected='{"reference": ["fedora"]}'),
            TestCase(
                name="list[str]", input=["reference=fedora"], expected='{"reference": ["fedora"]}'
            ),
            TestCase(
                name="dict[str,str]",
                input={"reference": "fedora"},
                expected='{"reference": ["fedora"]}',
            ),
        ]

        for case in cases:
            actual = api.prepare_filters(case.input)
            self.assertEqual(
                case.expected,
                actual,
                f"failed test {case.name} expected {case.expected}, actual {actual}",
            )

            if actual is not None:
                self.assertIsInstance(actual, str)

    def test_containerignore_404(self) -> None:
        actual = api.prepare_containerignore("/does/not/exists")
        self.assertListEqual([], actual)

    @patch.object(pathlib.Path, "exists", return_value=True)
    def test_containerignore_read(self, patch_exists) -> None:
        data = r"""# unittest

        #Ignore the logs directory
        logs/

        #Ignoring the password file
        passwords.txt

        #Ignoring git and cache folders
        .git
        .cache

        #Ignoring all the markdown and class files
        *.md
        **/*.class
        """

        with mock.patch("pathlib.Path.open", mock_open(read_data=data)):
            actual = api.prepare_containerignore(".")

        self.assertListEqual(
            actual, ["logs/", "passwords.txt", ".git", ".cache", "*.md", "**/*.class"]
        )
        patch_exists.assert_called_once_with()

    @patch.object(pathlib.Path, "exists", return_value=True)
    def test_containerignore_empty(self, patch_exists) -> None:
        data = r"""# unittest
        """

        patch_exists.return_value = True
        with mock.patch("pathlib.Path.open", mock_open(read_data=data)):
            actual = api.prepare_containerignore(".")

        self.assertListEqual(actual, [])
        patch_exists.assert_called_once_with()

    @mock.patch("pathlib.Path.parent", autospec=True)
    def test_containerfile_1(self, mock_parent) -> None:
        mock_parent.samefile.return_value = True
        actual = api.prepare_containerfile("/work", "/work/Dockerfile")
        self.assertEqual(actual, "Dockerfile")
        mock_parent.samefile.assert_called()

    @mock.patch("pathlib.Path.parent", autospec=True)
    def test_containerfile_2(self, mock_parent) -> None:
        mock_parent.samefile.return_value = True
        actual = api.prepare_containerfile(".", "Dockerfile")
        self.assertEqual(actual, "Dockerfile")
        mock_parent.samefile.assert_called()

    @mock.patch("shutil.copy2")
    def test_containerfile_copy(self, mock_copy) -> None:
        mock_copy.return_value = None

        with mock.patch.object(pathlib.Path, "parent") as mock_parent:
            mock_parent.samefile.return_value = False

            actual = api.prepare_containerfile("/work", "/home/Dockerfile")
            self.assertRegex(actual, r"\.containerfile\..*")

    def test_prepare_body_all_types(self) -> None:
        payload = {
            "String": "string",
            "Integer": 42,
            "Boolean": True,
            "Dictionary": {"key": "value"},
            "Tuple": (1, 2),
            "List": [1, 2],
        }
        actual = api.prepare_body(payload)
        self.assertEqual(actual, json.dumps(payload, sort_keys=True))

    def test_prepare_body_none(self) -> None:
        payload = {
            "String": "",
            "Integer": None,
            "Boolean": False,
            "Dictionary": dict(),
            "Tuple": tuple(),
            "List": list(),
        }
        actual = api.prepare_body(payload)
        self.assertEqual(actual, '{"Boolean": false}')

    def test_prepare_body_embedded(self) -> None:
        payload: dict[str, Any] = {
            "String": "",
            "Integer": None,
            "Boolean": False,
            "Dictionary": {"key": "value"},
            "Dictionary2": {"key": {"key2": None}},
            "Tuple": tuple(),
            "List": [None],
            "Set1": {"item1", "item2"},
            "Set2": {None},
        }
        actual = api.prepare_body(payload)
        actual_dict = json.loads(actual)

        # Because of the sets above we have to do some type dances to test results
        self.assertListEqual([*actual_dict], ["Boolean", "Dictionary", "Set1"])
        self.assertEqual(actual_dict["Boolean"], payload["Boolean"])
        self.assertDictEqual(actual_dict["Dictionary"], payload["Dictionary"])
        self.assertEqual(set(actual_dict["Set1"]), {"item1", "item2"})

    def test_prepare_body_dict_empty_string(self) -> None:
        payload = {"Dictionary": {"key1": "", "key2": {"key3": ""}, "key4": [], "key5": {}}}

        actual = api.prepare_body(payload)
        actual_dict = json.loads(actual)
        payload["Dictionary"].pop("key4")
        payload["Dictionary"].pop("key5")

        self.assertDictEqual(payload, actual_dict)

    def test_encode_auth_header(self):
        auth_config = {
            "username": "user",
            "password": "pass",
        }
        expected = b"eyJ1c2VybmFtZSI6ICJ1c2VyIiwgInBhc3N3b3JkIjogInBhc3MifQ=="
        actual = api.encode_auth_header(auth_config)
        self.assertEqual(expected, actual)


if __name__ == '__main__':
    unittest.main()



import unittest

from unittest.mock import patch, MagicMock

from podman.tests import utils


class TestPodmanVersion(unittest.TestCase):
    @patch('podman.tests.utils.subprocess.Popen')
    def test_podman_version(self, mock_popen):
        mock_proc = MagicMock()
        mock_proc.stdout.read.return_value = b'5.6.0'
        mock_popen.return_value.__enter__.return_value = mock_proc
        self.assertEqual(utils.podman_version(), (5, 6, 0))

    @patch('podman.tests.utils.subprocess.Popen')
    def test_podman_version_dev(self, mock_popen):
        mock_proc = MagicMock()
        mock_proc.stdout.read.return_value = b'5.6.0-dev'
        mock_popen.return_value.__enter__.return_value = mock_proc
        self.assertEqual(utils.podman_version(), (5, 6, 0))

    @patch('podman.tests.utils.subprocess.Popen')
    def test_podman_version_four_digits(self, mock_popen):
        mock_proc = MagicMock()
        mock_proc.stdout.read.return_value = b'5.6.0.1'
        mock_popen.return_value.__enter__.return_value = mock_proc
        self.assertEqual(utils.podman_version(), (5, 6, 0))

    @patch('podman.tests.utils.subprocess.Popen')
    def test_podman_version_release_candidate(self, mock_popen):
        mock_proc = MagicMock()
        mock_proc.stdout.read.return_value = b'5.6.0-rc1'
        mock_popen.return_value.__enter__.return_value = mock_proc
        self.assertEqual(utils.podman_version(), (5, 6, 0))

    @patch('podman.tests.utils.subprocess.Popen')
    def test_podman_version_none(self, mock_popen):
        mock_proc = MagicMock()
        mock_proc.stdout.read.return_value = b''
        mock_popen.return_value.__enter__.return_value = mock_proc
        with self.assertRaises(RuntimeError) as context:
            utils.podman_version()
        self.assertEqual(str(context.exception), "Unable to detect podman version. Got \"\"")



import random
import unittest

from podman import PodmanClient
from podman.errors import NotFound
from podman.tests.integration import base
from podman.tests.utils import PODMAN_VERSION


class VolumesIntegrationTest(base.IntegrationTest):
    def setUp(self):
        super().setUp()

        self.client = PodmanClient(base_url=self.socket_uri)
        self.addCleanup(self.client.close)

    def test_volume_crud(self):
        """Test Volume CRUD."""
        volume_name = f"volume_{random.getrandbits(160):x}"
        self.assertFalse(
            self.client.volumes.exists(volume_name), "Storage is corrupt from previous run"
        )

        with self.subTest("Create"):
            volume = self.client.volumes.create(volume_name)
            self.assertEqual(volume.name, volume_name)

        with self.subTest("Get"):
            actual = self.client.volumes.get(volume_name)
            self.assertDictEqual(actual.attrs, volume.attrs)

            self.assertTrue(self.client.volumes.exists(volume_name))

        with self.subTest("List"):
            report = self.client.volumes.list()
            names = [i.name for i in report]
            self.assertIn(volume_name, names)

        if PODMAN_VERSION >= (5, 6, 0):
            with self.subTest("Export"):
                archive_bytes = self.client.volumes.export_archive(volume_name)

            with self.subTest("Import"):
                self.client.volumes.import_archive(volume_name, archive_bytes)

        with self.subTest("Remove"):
            self.client.volumes.remove(volume_name, force=True)
            with self.assertRaises(NotFound):
                self.client.volumes.get(volume_name)

    def test_inspect_404(self):
        with self.assertRaises(NotFound):
            self.client.volumes.get("NoSuchVolume")


if __name__ == '__main__':
    unittest.main()



#   Copyright 2020 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""Images integration tests."""

import io
import os
import json
import platform
import tarfile
import tempfile
import types
import unittest
import random

import podman.tests.integration.base as base
from podman import PodmanClient
from podman.domain.images import Image
from podman.errors import APIError, ContainerError, ImageNotFound, PodmanError

# @unittest.skipIf(os.geteuid() != 0, 'Skipping, not running as root')


class ImagesIntegrationTest(base.IntegrationTest):
    """images call integration test"""

    def setUp(self):
        super().setUp()

        self.client = PodmanClient(base_url=self.socket_uri)
        self.addCleanup(self.client.close)

        self.test_manifest_name = "dummy:v1.2.3"

    def tearDown(self) -> None:
        if self.client.manifests.exists(self.test_manifest_name):
            self.client.manifests.remove(self.test_manifest_name)

    def test_image_crud(self):
        """Test Image CRUD.

        Notes:
            Written to maximize reuse of pulled image.
        """

        with self.subTest("Pull Alpine Image"):
            image = self.client.images.pull("quay.io/libpod/alpine", tag="latest")
            self.assertIsInstance(image, Image)
            self.assertIn("quay.io/libpod/alpine:latest", image.tags)
            self.assertTrue(self.client.images.exists(image.id))

        with self.subTest("Inspect Alpine Image"):
            image = self.client.images.get("quay.io/libpod/alpine")
            self.assertIsInstance(image, Image)
            self.assertIn("quay.io/libpod/alpine:latest", image.tags)

        with self.subTest("Retrieve Image history"):
            ids = [i["Id"] for i in image.history()]
            self.assertIn(image.id, ids)

        with self.subTest("Export Image to tarball (in memory)"):
            buffer = io.BytesIO()
            for chunk in image.save():
                buffer.write(chunk)
            buffer.seek(0, 0)

            with tarfile.open(fileobj=buffer, mode="r") as tar:
                items = tar.getnames()
            self.assertIn("manifest.json", items)
            self.assertIn("repositories", items)

        with self.subTest("List images"):
            image_list = self.client.images.list()
            self.assertTrue(
                any([i for i in image_list if "quay.io/libpod/alpine:latest" in i.tags]),
                f"pulled image 'quay.io/libpod/alpine:latest' not listed, {image_list}.",
            )

        with self.subTest("Tag and reload() Image"):
            self.assertTrue(image.tag("localhost/alpine", "unittest"))
            image.reload()
            self.assertIn("localhost/alpine:unittest", image.tags)

        with self.subTest("Delete Image"):
            actual = self.client.images.remove(image.id, force=True)
            deleted = [d["Deleted"] for d in actual if "Deleted" in d]
            self.assertIn(image.id, deleted)
            untagged = [d["Untagged"] for d in actual if "Untagged" in d]
            self.assertIn(image.tags[0], untagged)
            exit_code = [d["ExitCode"] for d in actual if "ExitCode" in d]
            self.assertIn(0, exit_code)

            with self.assertRaises(ImageNotFound):
                # verify image deleted before loading below
                self.client.images.get(image.id)

        with self.subTest("Load Image previously deleted"):
            buffer.seek(0, 0)
            new_image = next(iter(self.client.images.load(buffer.getvalue())))
            self.assertEqual(image.id, new_image.id)

        with self.subTest("Deleted unused Images"):
            actual = self.client.images.prune()
            deleted = [d.get("Deleted") for d in actual["ImagesDeleted"]]
            self.assertIn(image.id, deleted)
            self.assertGreater(actual["SpaceReclaimed"], 0)

        with self.subTest("Export Image to tarball (in memory) with named mode"):
            alpine_image = self.client.images.pull("quay.io/libpod/alpine", tag="latest")
            image_buffer = io.BytesIO()
            for chunk in alpine_image.save(named=True):
                image_buffer.write(chunk)
            image_buffer.seek(0, 0)

            with tarfile.open(fileobj=image_buffer, mode="r") as tar:
                items_in_tar = tar.getnames()
                # Check if repositories file is available in the tarball
                self.assertIn("repositories", items_in_tar)
                # Extract the 'repositories' file
                repositories_file = tar.extractfile("repositories")
                if repositories_file is not None:
                    # Check the content of the "repositories" file.
                    repositories_content = repositories_file.read().decode("utf-8")
                    # Check if "repositories" file contains the name of the Image (named).
                    self.assertTrue("alpine" in str(repositories_content))

    def test_search(self):
        # N/B: This is an infrequently used feature, that tends to flake a lot.
        # Just check that it doesn't throw an exception and move on.
        self.client.images.search("alpine")

    @unittest.skip("Needs Podman 3.1.0")
    def test_corrupt_load(self):
        with self.assertRaises(APIError) as e:
            next(self.client.images.load(b"This is a corrupt tarball"))
        self.assertIn("payload does not match", e.exception.explanation)

    def test_build(self):
        buffer = io.StringIO("""FROM scratch""")
        image, _ = self.client.images.build(fileobj=buffer)
        self.assertIsNotNone(image)
        self.assertIsNotNone(image.id)

    def test_build_cache(self):
        """Check build caching when enabled

        Build twice with caching enabled (default), then again with nocache
        """

        def look_for_cache(stream) -> bool:
            # Search for a line with contents "-> Using cache <image id>"
            uses_cache = False
            for line in stream:
                parsed = json.loads(line)['stream']
                if "Using cache" in parsed:
                    uses_cache = True
                    break
            return uses_cache

        label = str(random.getrandbits(32))
        buffer = io.StringIO(f"""FROM scratch\nLABEL test={label}""")
        image, _ = self.client.images.build(fileobj=buffer)
        buffer.seek(0)
        cached_image, stream = self.client.images.build(fileobj=buffer)
        self.assertTrue(look_for_cache(stream))
        self.assertEqual(
            cached_image.id,
            image.id,
            msg="Building twice with cache does not produce the same image id",
        )
        # Build again with disabled cache
        buffer.seek(0)
        uncached_image, stream = self.client.images.build(fileobj=buffer, nocache=True)
        self.assertFalse(look_for_cache(stream))
        self.assertNotEqual(
            uncached_image.id,
            image.id,
            msg="Building twice without cache produces the same image id",
        )

    def test_build_with_manifest(self):
        buffer = io.StringIO("""FROM quay.io/libpod/alpine_labels:latest""")

        self.assertFalse(self.client.manifests.exists(self.test_manifest_name))

        image, _ = self.client.images.build(fileobj=buffer, manifest=self.test_manifest_name)
        self.assertIsNotNone(image)
        self.assertIsNotNone(image.id)

        self.assertTrue(self.client.manifests.exists(self.test_manifest_name))

    def test_build_with_context(self):
        context = io.BytesIO()
        with tarfile.open(fileobj=context, mode="w") as tar:

            def add_file(name: str, content: str):
                binary_content = content.encode("utf-8")
                fileobj = io.BytesIO(binary_content)
                tarinfo = tarfile.TarInfo(name=name)
                tarinfo.size = len(binary_content)
                tar.addfile(tarinfo, fileobj)

            # Use a non-standard Dockerfile name to test the 'dockerfile' argument
            add_file(
                "MyDockerfile", ("FROM quay.io/libpod/alpine_labels:latest\nCOPY example.txt .\n")
            )
            add_file("example.txt", "This is an example file.\n")

        # Rewind to the start of the generated file so we can read it
        context.seek(0)

        with self.assertRaises(PodmanError):
            # If requesting a custom context, must provide the context as `fileobj`
            self.client.images.build(custom_context=True, path='invalid')

        with self.assertRaises(PodmanError):
            # If requesting a custom context, currently must specify the dockerfile name
            self.client.images.build(custom_context=True, fileobj=context)

        image, _ = self.client.images.build(
            fileobj=context,
            dockerfile="MyDockerfile",
            custom_context=True,
        )
        self.assertIsNotNone(image)
        self.assertIsNotNone(image.id)

    def test_build_with_secret(self):
        with tempfile.TemporaryDirectory() as context_dir:
            dockerfile_path = os.path.join(context_dir, "Dockerfile")
            with open(dockerfile_path, "w") as f:
                f.write("""
                FROM quay.io/libpod/alpine_labels:latest
                RUN --mount=type=secret,id=example cat /run/secrets/example > /output.txt
                """)

            secret_path = os.path.join(context_dir, "build-secret.txt")
            with open(secret_path, "w") as f:
                f.write("secret123")

            image, _ = self.client.images.build(
                path=context_dir,
                secrets=["id=example,src=build-secret.txt"],
                dockerfile="Dockerfile",
            )

        self.assertIsNotNone(image)
        self.assertIsNotNone(image.id)

        # Verify secret was passed and stored in file (NOT RECOMMENDED for real use cases)
        container_out = self.client.containers.run(
            image.id, command=["cat", "/output.txt"], remove=True, log_config={"Type": "json-file"}
        )
        self.assertIn(b"secret123", container_out)

        # Verify mounted secret file is not present in image
        with self.assertRaises(ContainerError) as exc:
            self.client.containers.run(
                image.id, command=["cat", "/run/secrets/example"], remove=True
            )
        self.assertIn("No such file or directory", b"".join(exc.exception.stderr).decode("utf-8"))

    @unittest.skipIf(platform.architecture()[0] == "32bit", "no 32-bit image available")
    def test_pull_stream(self):
        generator = self.client.images.pull("ubi8", tag="latest", stream=True)
        self.assertIsInstance(generator, types.GeneratorType)

    @unittest.skipIf(platform.architecture()[0] == "32bit", "no 32-bit image available")
    def test_pull_stream_decode(self):
        generator = self.client.images.pull("ubi8", tag="latest", stream=True, decode=True)
        self.assertIsInstance(generator, types.GeneratorType)

    def test_scp(self):
        with self.assertRaises(APIError) as e:
            next(
                self.client.images.scp(
                    source="randuser@fake.ip.addr:22::quay.io/libpod/alpine", quiet=False
                )
            )
        self.assertRegex(
            e.exception.explanation,
            r"failed to connect: dial tcp: lookup fake\.ip\.addr.+no such host",
        )



import unittest
import urllib.parse
from pathlib import Path
from unittest import mock
from unittest.mock import MagicMock

import requests_mock

from podman.domain.config import PodmanConfig, ServiceConnection
from podman import PodmanClient, tests
from podman.api.path_utils import get_runtime_dir, get_xdg_config_home


class PodmanClientTestCase(unittest.TestCase):
    """Test the PodmanClient() object."""

    opener = mock.mock_open(
        read_data="""
[containers]
  log_size_max = -1
  pids_limit = 2048
  userns_size = 65536

[engine]
  num_locks = 2048
  active_service = "testing"
  stop_timeout = 10
  [engine.service_destinations]
    [engine.service_destinations.production]
      uri = "ssh://root@localhost:22/run/podman/podman.sock"
      identity = "/home/root/.ssh/id_rsa"
    [engine.service_destinations.testing]
      uri = "ssh://qe@localhost:2222/run/podman/podman.sock"
      identity = "/home/qe/.ssh/id_rsa"

[network]
"""
    )

    def setUp(self) -> None:
        super().setUp()
        self.client = PodmanClient(base_url=tests.BASE_SOCK)

        def mocked_open(self, *args, **kwargs):
            return PodmanClientTestCase.opener(self, *args, **kwargs)

        self.mocked_open = mocked_open

    @mock.patch('requests.Session.close')
    def test_close(self, mock_close):
        self.client.close()

        mock_close.assert_called_once_with()

    @requests_mock.Mocker()
    def test_contextmanager(self, mock):
        body = {
            "host": {
                "arch": "amd65",
                "os": "linux",
            }
        }
        adapter = mock.get(tests.LIBPOD_URL + "/info", json=body)  # noqa: F841

        with PodmanClient(base_url=tests.BASE_SOCK) as client:
            actual = client.info()
        self.assertDictEqual(actual, body)
        self.assertIn("User-Agent", mock.last_request.headers)
        self.assertIn(
            "PodmanPy/", mock.last_request.headers["User-Agent"], mock.last_request.headers
        )

    def test_swarm(self):
        with PodmanClient(base_url=tests.BASE_SOCK) as client:
            with self.assertRaises(NotImplementedError):
                # concrete property
                _ = client.swarm

            with self.assertRaises(NotImplementedError):
                # aliased property
                _ = client.nodes

    def test_connect(self):
        with mock.patch.multiple(Path, open=self.mocked_open, exists=MagicMock(return_value=True)):
            with PodmanClient(connection="testing") as client:
                self.assertEqual(
                    client.api.base_url.geturl(),
                    "http+ssh://qe@localhost:2222/run/podman/podman.sock",
                )

            # Build path to support tests running as root or a user
            expected = Path(get_xdg_config_home()) / "containers" / "containers.conf"
            PodmanClientTestCase.opener.assert_called_with(expected, encoding="utf-8")

    def test_connect_404(self):
        with mock.patch.multiple(Path, open=self.mocked_open, exists=MagicMock(return_value=True)):
            with self.assertRaises(KeyError):
                _ = PodmanClient(connection="not defined")

    def test_connect_default(self):
        with mock.patch.multiple(Path, open=self.mocked_open, exists=MagicMock(return_value=True)):
            with PodmanClient() as client:
                expected = "http+unix://" + urllib.parse.quote_plus(
                    str(Path(get_runtime_dir()) / "podman" / "podman.sock")
                )
                self.assertEqual(client.api.base_url.geturl(), expected)

            # Build path to support tests running as root or a user
            expected = Path(get_xdg_config_home()) / "containers" / "containers.conf"
            PodmanClientTestCase.opener.assert_called_with(expected, encoding="utf-8")

    def test_connect_with_connection_file(self):
        mock_config = MagicMock(spec=PodmanConfig)
        mock_service = MagicMock(spec=ServiceConnection)
        mock_service.url.geturl.return_value = (
            "http+ssh://core@127.0.0.1:58468/run/user/501/podman/podman.sock"
        )
        mock_service.identity = "/Users/test/.local/share/containers/podman/machine/machine"
        mock_service.is_machine = True
        mock_config.active_service = mock_service

        with mock.patch('podman.client.PodmanConfig', return_value=mock_config):
            # Mock pathlib.Path.exists to return True for the identity file
            with mock.patch('pathlib.Path.exists', return_value=True):
                with PodmanClient() as client:
                    expected = "http+ssh://core@127.0.0.1:58468/run/user/501/podman/podman.sock"
                    self.assertEqual(client.api.base_url.geturl(), expected)


if __name__ == '__main__':
    unittest.main()



import unittest

import requests_mock

from podman import PodmanClient, tests
from podman.domain.volumes import Volume, VolumesManager

FIRST_VOLUME = {
    "CreatedAt": "1985-04-12T23:20:50.52Z",
    "Driver": "default",
    "Labels": {"BackupRequired": True},
    "Mountpoint": "/var/database",
    "Name": "dbase",
    "Scope": "local",
}


class VolumeTestCase(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()

        self.client = PodmanClient(base_url=tests.BASE_SOCK)

    def tearDown(self) -> None:
        super().tearDown()

        self.client.close()

    def test_id(self):
        actual = Volume(attrs={"Name": "dbase"})
        self.assertEqual(actual.id, "dbase")

    @requests_mock.Mocker()
    def test_remove(self, mock):
        adapter = mock.delete(tests.LIBPOD_URL + "/volumes/dbase?force=True", status_code=204)
        vol_manager = VolumesManager(self.client.api)
        volume = vol_manager.prepare_model(attrs=FIRST_VOLUME)

        volume.remove(force=True)
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_inspect(self, mock):
        mock.get(tests.LIBPOD_URL + "/volumes/dbase/json?tlsVerify=False", json=FIRST_VOLUME)
        vol_manager = VolumesManager(self.client.api)
        actual = vol_manager.prepare_model(attrs=FIRST_VOLUME)
        self.assertEqual(actual.inspect(tls_verify=False)["Mountpoint"], "/var/database")


if __name__ == '__main__':
    unittest.main()



"""Secrets integration tests."""

import random
import unittest
import uuid

from podman import PodmanClient
from podman.errors import NotFound
from podman.tests.integration import base


class SecretsIntegrationTest(base.IntegrationTest):
    """Secrets call integration test"""

    def setUp(self):
        super().setUp()

        self.client = PodmanClient(base_url=self.socket_uri)
        self.addCleanup(self.client.close)

    def test_secret_crud(self):
        """Test Secret CRUD."""

        random_string = f"{random.getrandbits(160):x}"
        secret_payload = uuid.uuid4().bytes

        with self.subTest("Create"):
            secret = self.client.secrets.create(f"secret_{random_string}", secret_payload)
            self.assertGreater(len(secret.id), 0)
            self.assertGreater(len(secret.name), 0)
            self.assertTrue(self.client.secrets.exists(secret.id))

        with self.subTest("Inspect"):
            actual = self.client.secrets.get(secret.id)
            self.assertEqual(secret.id, actual.id)

        self.assertTrue(self.client.secrets.exists(secret.name))

        with self.subTest("List"):
            report = self.client.secrets.list()
            ids = [i.id for i in report]
            self.assertIn(secret.id, ids)

        with self.subTest("Delete"):
            actual.remove()

            with self.assertRaises(NotFound):
                self.client.secrets.get(secret.id)


if __name__ == '__main__':
    unittest.main()



import random
import unittest

from podman import PodmanClient
from podman.errors import NotFound
from podman.tests.integration import base


class PodsIntegrationTest(base.IntegrationTest):
    """Pods Integration tests."""

    def setUp(self):
        super().setUp()

        self.client = PodmanClient(base_url=self.socket_uri)
        self.addCleanup(self.client.close)

        self.alpine_image = self.client.images.pull("quay.io/libpod/alpine", tag="latest")
        self.pod_name = f"pod_{random.getrandbits(160):x}"

        for container in self.client.containers.list():
            container.remove(force=True)

    def tearDown(self):
        if self.client.pods.exists(self.pod_name):
            self.client.pods.remove(self.pod_name)
        super().tearDown()

    def test_pod_crud(self):
        """Test Pod CRUD."""
        with self.subTest("Create no_infra"):
            pod = self.client.pods.create(
                self.pod_name,
                labels={
                    "unittest": "true",
                },
                no_infra=True,
            )
            self.assertEqual(self.pod_name, pod.name)
            self.assertTrue(self.client.pods.exists(pod.id))

        with self.subTest("Inspect"):
            actual = self.client.pods.get(pod.id)
            self.assertEqual(actual.name, pod.name)
            self.assertNotIn("Containers", actual.attrs)

        with self.subTest("Exists"):
            self.assertTrue(self.client.pods.exists(self.pod_name))

        # TODO Need method for deterministic prune...
        # with self.subTest("Prune"):
        # report = self.client.pods.prune()
        # self.assertIn("PodsDeleted", report)
        # self.assertIn(actual.id, report["PodsDeleted"])
        #
        # with self.assertRaises(NotFound):
        #     pod.reload()
        #
        # For now, delete pod explicitly
        with self.subTest("Delete"):
            pod.remove(force=True)
            with self.assertRaises(NotFound):
                pod.reload()

    def test_pod_crud_infra(self):
        """Test Pod CRUD with infra container."""

        with self.subTest("Create with infra"):
            pod = self.client.pods.create(
                self.pod_name,
                labels={
                    "unittest": "true",
                },
            )
            self.assertEqual(self.pod_name, pod.name)

        with self.subTest("Inspect"):
            actual = self.client.pods.get(pod.id)
            self.assertEqual(actual.name, pod.name)
            self.assertIn("Containers", actual.attrs)
            self.assertEqual(actual.attrs["State"], "Created")

        with self.subTest("Add container"):
            container = self.client.containers.create(self.alpine_image, command=["ls"], pod=actual)
            actual = self.client.pods.get(pod.id)

            ids = {c["Id"] for c in actual.attrs["Containers"]}
            self.assertIn(container.id, ids)

        with self.subTest("List"):
            pods = self.client.pods.list()
            self.assertGreaterEqual(len(pods), 1)

            ids = {p.id for p in pods}
            self.assertIn(actual.id, ids)

        with self.subTest("Delete"):
            pod.remove(force=True)
            with self.assertRaises(NotFound):
                pod.reload()

    def test_ps(self):
        pod = self.client.pods.create(
            self.pod_name,
            labels={
                "unittest": "true",
            },
            no_infra=True,
        )
        self.assertTrue(self.client.pods.exists(pod.id))
        self.client.containers.create(
            self.alpine_image, command=["top"], detach=True, tty=True, pod=pod
        )
        pod.start()
        pod.reload()

        with self.subTest("top"):
            # this is the API top call not the
            # top command running in the container
            procs = pod.top()

            self.assertGreater(len(procs["Processes"]), 0)
            self.assertGreater(len(procs["Titles"]), 0)

        with self.subTest("stats"):
            report = self.client.pods.stats(all=True)
            self.assertGreaterEqual(len(report), 1)

        with self.subTest("Stop/Start"):
            pod.stop()
            pod.reload()
            self.assertIn(pod.attrs["State"], ("Stopped", "Exited"))

            pod.start()
            pod.reload()
            self.assertEqual(pod.attrs["State"], "Running")

        with self.subTest("Restart"):
            pod.stop()
            pod.restart()
            pod.reload()
            self.assertEqual(pod.attrs["State"], "Running")

        with self.subTest("Pause/Unpause"):
            pod.pause()
            pod.reload()
            self.assertEqual(pod.attrs["State"], "Paused")

            pod.unpause()
            pod.reload()
            self.assertEqual(pod.attrs["State"], "Running")

        pod.stop()


if __name__ == '__main__':
    unittest.main()



import unittest
from contextlib import suppress

from podman import PodmanClient
from podman.errors import APIError, ImageNotFound
from podman.tests.integration import base


# @unittest.skipIf(os.geteuid() != 0, 'Skipping, not running as root')


class ManifestsIntegrationTest(base.IntegrationTest):
    def setUp(self):
        super().setUp()

        self.client = PodmanClient(base_url=self.socket_uri)
        self.addCleanup(self.client.close)

        self.alpine_image = self.client.images.pull("quay.io/libpod/alpine", tag="latest")
        self.invalid_manifest_name = "InvalidManifestName"

    def tearDown(self) -> None:
        if self.client.images.exists(self.invalid_manifest_name):
            self.client.images.remove(self.invalid_manifest_name, force=True)

        self.client.images.remove(self.alpine_image, force=True)
        with suppress(ImageNotFound):
            self.client.images.remove("localhost/unittest/alpine", force=True)

    def test_manifest_crud(self):
        """Test Manifest CRUD."""

        self.assertFalse(
            self.client.manifests.exists("localhost/unittest/alpine"),
            "Image store is corrupt from previous run",
        )

        with self.subTest("Create"):
            manifest = self.client.manifests.create(
                "localhost/unittest/alpine", ["quay.io/libpod/alpine:latest"]
            )
            self.assertEqual(len(manifest.attrs["manifests"]), 1, manifest.attrs)
            self.assertTrue(self.client.manifests.exists(manifest.names), manifest.id)

            with self.assertRaises(APIError):
                self.client.manifests.create("123456!@#$%^")

        with self.subTest("Add"):
            manifest.add([self.alpine_image])
            self.assertIsNotNone(manifest.attrs["manifests"])
            self.assertIsInstance(manifest.attrs["manifests"], list)

            self.assertTrue(
                any([d for d in self.alpine_image.attrs["RepoDigests"] if manifest.id in d]),
                f'{manifest.id} not in any {self.alpine_image.attrs["RepoDigests"]}',
            )

        with self.subTest("Inspect"):
            actual = self.client.manifests.get("quay.io/libpod/alpine:latest")
            self.assertEqual(actual.id, manifest.id)

            actual = self.client.manifests.get(manifest.name)
            self.assertEqual(actual.id, manifest.id)

            self.assertEqual(actual.version, 2)

        with self.subTest("Remove digest"):
            manifest.remove(self.alpine_image.attrs["RepoDigests"][0])
            self.assertEqual(manifest.attrs["manifests"], [])

    def test_create_409(self):
        """Test that invalid Image names are caught and not corrupt storage."""
        with self.assertRaises(APIError):
            self.client.manifests.create(self.invalid_manifest_name)


if __name__ == '__main__':
    unittest.main()



import unittest
import urllib.parse
import json
import os
import tempfile
from pathlib import Path
from unittest import mock
from unittest.mock import MagicMock
from podman.domain.config import PodmanConfig


class PodmanConfigTestCaseDefault(unittest.TestCase):
    def setUp(self):
        self.temp_dir = tempfile.mkdtemp()

        # Data to be written to the JSON file
        self.data_json = """
{
  "Connection": {
    "Default": "testing_json",
    "Connections": {
      "testing_json": {
        "URI": "ssh://qe@localhost:2222/run/podman/podman.sock",
        "Identity": "/home/qe/.ssh/id_rsa"
      },
      "production": {
        "URI": "ssh://root@localhost:22/run/podman/podman.sock",
        "Identity": "/home/root/.ssh/id_rsajson"
      }
    }
  },
  "Farm": {}
}
"""

        # Data to be written to the TOML file
        self.data_toml = """
[containers]
  log_size_max = -1
  pids_limit = 2048
  userns_size = 65536

[engine]
  num_locks = 2048
  active_service = "testing"
  stop_timeout = 10
  [engine.service_destinations]
    [engine.service_destinations.production]
      uri = "ssh://root@localhost:22/run/podman/podman.sock"
      identity = "/home/root/.ssh/id_rsa"
    [engine.service_destinations.testing]
      uri = "ssh://qe@localhost:2222/run/podman/podman.sock"
      identity = "/home/qe/.ssh/id_rsa"

[network]
"""

        # Define the file path
        self.path_json = os.path.join(self.temp_dir, 'podman-connections.json')
        self.path_toml = os.path.join(self.temp_dir, 'containers.conf')

        # Write data to the JSON file
        j_data = json.loads(self.data_json)
        with open(self.path_json, 'w+') as file_json:
            json.dump(j_data, file_json)

        # Write data to the TOML file
        with open(self.path_toml, 'w+') as file_toml:
            # toml.dump(self.data_toml, file_toml)
            file_toml.write(self.data_toml)

    def test_connections(self):
        config = PodmanConfig("@@is_test@@" + self.temp_dir)

        self.assertEqual(config.active_service.id, "testing_json")

        expected = urllib.parse.urlparse("ssh://qe@localhost:2222/run/podman/podman.sock")
        self.assertEqual(config.active_service.url, expected)
        self.assertEqual(config.services["production"].identity, Path("/home/root/.ssh/id_rsajson"))


class PodmanConfigTestCaseTOML(unittest.TestCase):
    opener = mock.mock_open(
        read_data="""
[containers]
  log_size_max = -1
  pids_limit = 2048
  userns_size = 65536

[engine]
  num_locks = 2048
  active_service = "testing"
  stop_timeout = 10
  [engine.service_destinations]
    [engine.service_destinations.production]
      uri = "ssh://root@localhost:22/run/podman/podman.sock"
      identity = "/home/root/.ssh/id_rsa"
    [engine.service_destinations.testing]
      uri = "ssh://qe@localhost:2222/run/podman/podman.sock"
      identity = "/home/qe/.ssh/id_rsa"

[network]
"""
    )

    def setUp(self) -> None:
        super().setUp()

        def mocked_open(self, *args, **kwargs):
            return PodmanConfigTestCaseTOML.opener(self, *args, **kwargs)

        self.mocked_open = mocked_open

    def test_connections(self):
        with mock.patch.multiple(Path, open=self.mocked_open, exists=MagicMock(return_value=True)):
            config = PodmanConfig("/home/developer/containers.conf")

            self.assertEqual(config.active_service.id, "testing")

            expected = urllib.parse.urlparse("ssh://qe@localhost:2222/run/podman/podman.sock")
            self.assertEqual(config.active_service.url, expected)
            self.assertEqual(config.services["production"].identity, Path("/home/root/.ssh/id_rsa"))

            PodmanConfigTestCaseTOML.opener.assert_called_with(
                Path("/home/developer/containers.conf"), encoding='utf-8'
            )


class PodmanConfigTestCaseJSON(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()

        self.temp_dir = tempfile.mkdtemp()
        self.data = """
{
  "Connection": {
    "Default": "testing",
    "Connections": {
      "testing": {
        "URI": "ssh://qe@localhost:2222/run/podman/podman.sock",
        "Identity": "/home/qe/.ssh/id_rsa"
      },
      "production": {
        "URI": "ssh://root@localhost:22/run/podman/podman.sock",
        "Identity": "/home/root/.ssh/id_rsa"
      }
    }
  },
  "Farm": {}
}
"""

        self.path = os.path.join(self.temp_dir, 'podman-connections.json')
        # Write data to the JSON file
        data = json.loads(self.data)
        with open(self.path, 'w+') as file:
            json.dump(data, file)

    def test_connections(self):
        config = PodmanConfig(self.path)

        self.assertEqual(config.active_service.id, "testing")

        expected = urllib.parse.urlparse("ssh://qe@localhost:2222/run/podman/podman.sock")
        self.assertEqual(config.active_service.url, expected)
        self.assertEqual(config.services["production"].identity, Path("/home/root/.ssh/id_rsa"))


if __name__ == '__main__':
    unittest.main()



import unittest

import requests_mock

from podman import PodmanClient, tests


CONTAINER = {
    "Id": "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
    "Name": "quay.io/fedora:latest",
    "Image": "eloquent_pare",
    "State": {"Status": "running"},
}


class PodmanResourceTestCase(unittest.TestCase):
    """Test PodmanResource area of concern."""

    def setUp(self) -> None:
        super().setUp()

        self.client = PodmanClient(base_url=tests.BASE_SOCK)

    def tearDown(self) -> None:
        super().tearDown()

        self.client.close()

    @requests_mock.Mocker()
    def test_reload_with_compatible_options(self, mock):
        """Test that reload uses the correct endpoint."""

        # Mock the get() call
        mock.get(
            f"{tests.LIBPOD_URL}/"
            f"containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/json",
            json=CONTAINER,
        )

        # Mock the reload() call
        mock.get(
            f"{tests.LIBPOD_URL}/"
            f"containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/json",
            json=CONTAINER,
        )

        # Mock the reload(compatible=False) call
        mock.get(
            f"{tests.LIBPOD_URL}/"
            f"containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/json",
            json=CONTAINER,
        )

        # Mock the reload(compatible=True) call
        mock.get(
            f"{tests.COMPATIBLE_URL}/"
            f"containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/json",
            json=CONTAINER,
        )

        container = self.client.containers.get(
            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd"
        )
        container.reload()
        container.reload(compatible=False)
        container.reload(compatible=True)

        self.assertEqual(len(mock.request_history), 4)
        for i in range(3):
            self.assertEqual(
                mock.request_history[i].url,
                tests.LIBPOD_URL.lower()
                + "/containers/"
                + "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/json",
            )
        self.assertEqual(
            mock.request_history[3].url,
            tests.COMPATIBLE_URL.lower()
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/json",
        )


if __name__ == '__main__':
    unittest.main()



import unittest

import requests_mock

from podman import PodmanClient, tests


class SystemTestCase(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()

        self.client = PodmanClient(base_url=tests.BASE_SOCK)

    def tearDown(self) -> None:
        super().tearDown()

        self.client.close()

    @requests_mock.Mocker()
    def test_df(self, mock):
        body = {
            "Containers": [
                {"ContainerID": "f1fb3564c202"},
                {"ContainerID": "779afab684c7"},
            ],
            "Images": [
                {"ImageID": "118cc2c68ef5"},
                {"ImageID": "a6b4a8255f9e"},
            ],
            "Volumes": [
                {"VolumeName": "27df59163be8"},
                {"VolumeName": "77a83a10f86e"},
            ],
        }

        mock.get(
            tests.LIBPOD_URL + "/system/df",
            json=body,
        )

        actual = self.client.df()
        self.assertDictEqual(actual, body)

    @requests_mock.Mocker()
    def test_info(self, mock):
        body = {
            "host": {
                "arch": "amd65",
                "os": "linux",
            }
        }
        mock.get(tests.LIBPOD_URL + "/info", json=body)

        actual = self.client.info()
        self.assertDictEqual(actual, body)

    @requests_mock.Mocker()
    def test_ping(self, mock):
        mock.head(tests.LIBPOD_URL + "/_ping")
        self.assertTrue(self.client.ping())

    @requests_mock.Mocker()
    def test_version(self, mock):
        body = {
            "APIVersion": "3.0.0",
            "MinAPIVersion": "3.0.0",
            "Arch": "amd64",
            "Os": "linux",
        }
        mock.get(tests.LIBPOD_URL + "/version", json=body)
        self.assertDictEqual(self.client.version(), body)


if __name__ == '__main__':
    unittest.main()



import unittest

import requests_mock

from podman import PodmanClient, tests
from podman.domain.networks import Network
from podman.domain.networks_manager import NetworksManager

FIRST_NETWORK = {
    "Name": "podman",
    "Id": "2f259bab93aaaaa2542ba43ef33eb990d0999ee1b9924b557b7be53c0b7a1bb9",
    "Created": "2021-03-01T09:18:37.491308364-07:00",
    "Scope": "local",
    "Driver": "bridge",
    "EnableIPv6": False,
    "IPAM": {
        "Driver": "default",
        "Options": {},
        "Config": [{"Subnet": "10.88.0.0/16", "Gateway": "10.88.0.1"}],
    },
    "Internal": False,
    "Attachable": False,
    "Ingress": False,
    "ConfigFrom": {"Network": ""},
    "ConfigOnly": False,
    "Containers": {},
    "Options": {},
    "Labels": {},
}

FIRST_NETWORK_LIBPOD = {
    "name": "podman",
    "id": "2f259bab93aaaaa2542ba43ef33eb990d0999ee1b9924b557b7be53c0b7a1bb9",
    "driver": "bridge",
    "network_interface": "libpod_veth0",
    "created": "2022-01-28T09:18:37.491308364-07:00",
    "subnets": [
        {
            "subnet": "10.11.12.0/24",
            "gateway": "10.11.12.1",
            "lease_range": {
                "start_ip": "10.11.12.1",
                "end_ip": "10.11.12.63",
            },
        }
    ],
    "ipv6_enabled": False,
    "internal": False,
    "dns_enabled": False,
    "labels": {},
    "options": {},
    "ipam_options": {},
}


class NetworkTestCase(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()

        self.client = PodmanClient(base_url=tests.BASE_SOCK)
        self.addCleanup(self.client.close)

    def test_id(self):
        expected = {"Id": "1cf06390-709d-4ffa-a054-c3083abe367c"}
        actual = Network(attrs=expected)
        self.assertEqual(actual.id, expected["Id"])

        actual = Network(attrs={"name": "database"})
        self.assertEqual(
            actual.id, "3549b0028b75d981cdda2e573e9cb49dedc200185876df299f912b79f69dabd8"
        )

    def test_name(self):
        actual = Network(attrs={"Name": "database"})
        self.assertEqual(actual.name, "database")

        actual = Network({"name": "database"})
        self.assertEqual(actual.name, "database")

    @requests_mock.Mocker()
    def test_remove(self, mock):
        adapter = mock.delete(
            tests.LIBPOD_URL + "/networks/podman?force=True",
            status_code=204,
            json={"Name": "podman", "Err": None},
        )
        net_manager = NetworksManager(client=self.client.api)
        net = net_manager.prepare_model(attrs=FIRST_NETWORK)

        net.remove(force=True)
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_connect(self, mock):
        adapter = mock.post(tests.LIBPOD_URL + "/networks/podman/connect")
        net = Network(attrs=FIRST_NETWORK, client=self.client.api)

        net.connect(
            "podman_ctnr",
            aliases=["production"],
            ipv4_address="172.16.0.1",
        )
        self.assertEqual(adapter.call_count, 1)
        self.assertDictEqual(
            adapter.last_request.json(),
            {
                'Container': 'podman_ctnr',
                "EndpointConfig": {
                    'Aliases': ['production'],
                    'IPAMConfig': {'IPv4Address': '172.16.0.1'},
                    'IPAddress': '172.16.0.1',
                    'NetworkID': '2f259bab93aaaaa2542ba43ef33eb990d0999ee1b9924b557b7be53c0b7a1bb9',
                },
            },
        )

    @requests_mock.Mocker()
    def test_disconnect(self, mock):
        adapter = mock.post(tests.LIBPOD_URL + "/networks/podman/disconnect")
        net = Network(attrs=FIRST_NETWORK, client=self.client.api)

        net.disconnect("podman_ctnr", force=True)
        self.assertEqual(adapter.call_count, 1)
        self.assertDictEqual(
            adapter.last_request.json(),
            {
                'Container': 'podman_ctnr',
                "Force": True,
            },
        )


if __name__ == '__main__':
    unittest.main()



import podman.tests.integration.base as base
from podman import PodmanClient

# @unittest.skipIf(os.geteuid() != 0, 'Skipping, not running as root')


class ContainersExecIntegrationTests(base.IntegrationTest):
    """Containers integration tests for exec"""

    def setUp(self):
        super().setUp()

        self.client = PodmanClient(base_url=self.socket_uri)
        self.addCleanup(self.client.close)

        self.alpine_image = self.client.images.pull("quay.io/libpod/alpine", tag="latest")
        self.containers = []

    def tearDown(self):
        for container in self.containers:
            container.remove(force=True)

    def test_container_exec_run(self):
        """Test any command that will return code 0 and no output"""
        container = self.client.containers.create(self.alpine_image, command=["top"], detach=True)
        container.start()
        error_code, stdout = container.exec_run("echo hello")

        self.assertEqual(error_code, 0)
        self.assertEqual(stdout, b'\x01\x00\x00\x00\x00\x00\x00\x06hello\n')

    def test_container_exec_run_errorcode(self):
        """Test a failing command with stdout and stderr in a single bytestring"""
        container = self.client.containers.create(self.alpine_image, command=["top"], detach=True)
        container.start()
        error_code, output = container.exec_run("ls nonexistent")

        self.assertEqual(error_code, 1)
        self.assertEqual(
            output, b"\x02\x00\x00\x00\x00\x00\x00+ls: nonexistent: No such file or directory\n"
        )

    def test_container_exec_run_demux(self):
        """Test a failing command with stdout and stderr in a bytestring tuple"""
        container = self.client.containers.create(self.alpine_image, command=["top"], detach=True)
        container.start()
        error_code, output = container.exec_run("ls nonexistent", demux=True)

        self.assertEqual(error_code, 1)
        self.assertEqual(output[0], None)
        self.assertEqual(output[1], b"ls: nonexistent: No such file or directory\n")

    def test_container_exec_run_stream(self):
        """Test streaming the output from a long running command."""
        container = self.client.containers.create(self.alpine_image, command=["top"], detach=True)
        container.start()

        command = [
            '/bin/sh',
            '-c',
            'echo 0 ; sleep .1 ; echo 1 ; sleep .1 ; echo 2 ; sleep .1 ;',
        ]
        error_code, output = container.exec_run(command, stream=True)

        self.assertEqual(error_code, None)
        self.assertEqual(
            list(output),
            [
                b'0\n',
                b'1\n',
                b'2\n',
            ],
        )

    def test_container_exec_run_stream_demux(self):
        """Test streaming the output from a long running command with demux enabled."""
        container = self.client.containers.create(self.alpine_image, command=["top"], detach=True)
        container.start()

        command = [
            '/bin/sh',
            '-c',
            'echo 0 ; >&2 echo 1 ; sleep .1 ; '
            + 'echo 2 ; >&2 echo 3 ; sleep .1 ; '
            + 'echo 4 ; >&2 echo 5 ; sleep .1 ;',
        ]
        error_code, output = container.exec_run(command, stream=True, demux=True)

        self.assertEqual(error_code, None)
        self.assertEqual(
            list(output),
            [
                (b'0\n', None),
                (None, b'1\n'),
                (b'2\n', None),
                (None, b'3\n'),
                (b'4\n', None),
                (None, b'5\n'),
            ],
        )

    def test_container_exec_run_stream_detach(self):
        """Test streaming the output from a long running command with detach enabled."""
        container = self.client.containers.create(self.alpine_image, command=["top"], detach=True)
        container.start()

        command = [
            '/bin/sh',
            '-c',
            'echo 0 ; sleep .1 ; echo 1 ; sleep .1 ; echo 2 ; sleep .1 ;',
        ]
        error_code, output = container.exec_run(command, stream=True, detach=True)

        # Detach should make the ``exec_run`` ignore the ``stream`` flag so we will
        # assert against the standard, non-streaming behavior.
        self.assertEqual(error_code, 0)
        # The endpoint should return immediately, before we are able to actually
        # get any of the output.
        self.assertEqual(
            output,
            b'\n',
        )



import base64
import io
import json
import unittest

try:
    # Python >= 3.10
    from collections.abc import Iterable
except ImportError:
    # Python < 3.10
    from collections.abc import Iterable

import requests_mock

from podman import PodmanClient, tests
from podman.domain.containers import Container
from podman.domain.containers_manager import ContainersManager
from podman.errors import APIError, NotFound

FIRST_CONTAINER = {
    "Id": "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
    "Image": "quay.io/fedora:latest",
    "Name": "evil_ptolemy",
}


class ContainersTestCase(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()

        self.client = PodmanClient(base_url=tests.BASE_SOCK)

    def tearDown(self) -> None:
        super().tearDown()

        self.client.close()

    @requests_mock.Mocker()
    def test_remove(self, mock):
        adapter = mock.delete(
            tests.LIBPOD_URL + "/containers/"
            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd?v=True&force=True",
            status_code=204,
        )
        manager = ContainersManager(self.client.api)
        container = manager.prepare_model(attrs=FIRST_CONTAINER)
        container.remove(v=True, force=True)
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_rename(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/rename",
            status_code=204,
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        container.rename("good_galileo")
        self.assertEqual(container.attrs["Name"], "good_galileo")
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_rename_type_error(self, mock):
        container = Container(
            attrs={"ID": "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd"}
        )
        with self.assertRaises(TypeError):
            container.rename()

    @requests_mock.Mocker()
    def test_restart(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL + "/containers/"
            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/restart?timeout=10",
            status_code=204,
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        container.restart(timeout=10)
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_start_dkeys(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL + "/containers/"
            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/start"
            "?detachKeys=%5Ef%5Eu",
            status_code=204,
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        container.start(detach_keys="^f^u")
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_start(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/start",
            status_code=204,
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        container.start()
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_init(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/init",
            status_code=204,
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        container.init()
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_stats(self, mock):
        stream = [
            {
                "Error": None,
                "Stats": [
                    {
                        "ContainerId": (
                            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd"
                        ),
                        "Name": "evil_ptolemy",
                        "CPU": 1000.0,
                    }
                ],
            }
        ]
        buffer = io.StringIO()
        for entry in stream:
            buffer.write(json.JSONEncoder().encode(entry))
            buffer.write("\n")

        adapter = mock.get(
            tests.LIBPOD_URL + "/containers/stats"
            "?containers=87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd"
            "&stream=True",
            text=buffer.getvalue(),
        )

        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        stats = container.stats(decode=True)
        self.assertIsInstance(stats, Iterable)

        for entry in stats:
            self.assertIsInstance(entry, dict)
            for stat in entry["Stats"]:
                self.assertEqual(
                    stat["ContainerId"],
                    "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd",
                )
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_stop(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL + "/containers/"
            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/stop"
            "?all=True&timeout=10.0",
            status_code=204,
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        container.stop(all=True, timeout=10.0)
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_stop_304(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/stop",
            json={
                "cause": "container already stopped",
                "message": "container already stopped",
                "response": 304,
            },
            status_code=304,
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        with self.assertRaises(APIError):
            container.stop()
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_unpause(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL + "/containers/"
            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/unpause",
            status_code=204,
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        container.unpause()
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_pause(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/pause",
            status_code=204,
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        container.pause()
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_wait(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/wait",
            status_code=200,
            json={"StatusCode": 0},
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        container.wait()
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_wait_condition_interval(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/wait",
            status_code=200,
            json={"StatusCode": 0},
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        container.wait(condition="exited", interval=1)
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_diff(self, mock):
        payload = [
            {"Path": "modified", "Kind": 0},
            {"Path": "added", "Kind": 1},
            {"Path": "deleted", "Kind": 2},
        ]
        adapter = mock.get(
            tests.LIBPOD_URL + "/containers/"
            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/changes",
            json=payload,
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        actual = container.diff()
        self.assertListEqual(actual, payload)
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_diff_404(self, mock):
        adapter = mock.get(
            tests.LIBPOD_URL + "/containers/"
            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/changes",
            json={
                "cause": "Container not found.",
                "message": "Container not found.",
                "response": 404,
            },
            status_code=404,
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        with self.assertRaises(NotFound):
            container.diff()
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_export(self, mock):
        tarball = b'Yet another weird tarball...'
        body = io.BytesIO(tarball)
        adapter = mock.get(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/export",
            body=body,
        )

        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        with io.BytesIO() as fd:
            for chunk in container.export():
                fd.write(chunk)
            self.assertEqual(fd.getbuffer(), tarball)
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_get_archive(self, mock):
        tarball = b'Yet another weird tarball...'
        body = io.BytesIO(tarball)

        header_value = {
            "name": "/etc/motd",
            "size": len(tarball),
            "mode": 0o444,
            "mtime": "20210309T12:49:0205:00",
        }
        encoded_value = base64.urlsafe_b64encode(json.dumps(header_value).encode("utf8"))

        adapter = mock.get(
            tests.LIBPOD_URL + "/containers/"
            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/archive"
            "?path=/etc/motd",
            body=body,
            headers={"x-docker-container-path-stat": encoded_value.decode("utf8")},
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        actual = container.get_archive("/etc/motd")
        self.assertEqual(len(actual), 2)

        self.assertEqual(actual[1]["name"], "/etc/motd")

        with io.BytesIO() as fd:
            for chunk in actual[0]:
                fd.write(chunk)
            self.assertEqual(fd.getbuffer(), tarball)
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_commit(self, mock):
        post_adapter = mock.post(
            tests.LIBPOD_URL + "/commit"
            "?author=redhat&changes=ADD+%2fetc%2fmod&comment=This+is+a+unittest"
            "&container=87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd&format=docker"
            "&pause=True&repo=quay.local&tag=unittest",
            status_code=201,
            json={"Id": "d2459aad75354ddc9b5b23f863786e279637125af6ba4d4a83f881866b3c903f"},
        )
        get_adapter = mock.get(
            tests.LIBPOD_URL
            + "/images/d2459aad75354ddc9b5b23f863786e279637125af6ba4d4a83f881866b3c903f/json",
            json={"Id": "d2459aad75354ddc9b5b23f863786e279637125af6ba4d4a83f881866b3c903f"},
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)

        image = container.commit(
            repository="quay.local",
            tag="unittest",
            author="redhat",
            changes=["ADD /etc/mod"],
            comment="This is a unittest",
            format="docker",
            message="This is a unittest",
            pause=True,
        )
        self.assertEqual(
            image.id, "d2459aad75354ddc9b5b23f863786e279637125af6ba4d4a83f881866b3c903f"
        )
        self.assertTrue(post_adapter.called_once)
        self.assertTrue(get_adapter.called_once)

    @requests_mock.Mocker()
    def test_put_archive(self, mock):
        adapter = mock.put(
            tests.LIBPOD_URL + "/containers/"
            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/archive"
            "?path=%2fetc%2fmotd",
            status_code=200,
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)

        tarball = b'Yet another weird tarball...'
        body = io.BytesIO(tarball)
        actual = container.put_archive(path="/etc/motd", data=body.getvalue())
        self.assertTrue(actual)
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_put_archive_404(self, mock):
        adapter = mock.put(
            tests.LIBPOD_URL + "/containers/"
            "87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/archive"
            "?path=deadbeef",
            status_code=404,
            json={
                "cause": "Container not found.",
                "message": "Container not found.",
                "response": 404,
            },
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)

        tarball = b'Yet another weird tarball...'
        body = io.BytesIO(tarball)
        actual = container.put_archive(path="deadbeef", data=body.getvalue())
        self.assertFalse(actual)
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_top(self, mock):
        body = {
            "Processes": [
                [
                    'jhonce',
                    '2417',
                    '2274',
                    '0',
                    'Mar01',
                    '?',
                    '00:00:01',
                    '/usr/bin/ssh-agent /bin/sh -c exec -l /bin/bash -c "/usr/bin/gnome-session"',
                ],
                ['jhonce', '5544', '3522', '0', 'Mar01', 'pts/1', '00:00:02', '-bash'],
                ['jhonce', '6140', '3522', '0', 'Mar01', 'pts/2', '00:00:00', '-bash'],
            ],
            "Titles": ["UID", "PID", "PPID", "C", "STIME", "TTY", "TIME CMD"],
        }
        adapter = mock.get(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/top",
            json=body,
        )
        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        actual = container.top()
        self.assertDictEqual(actual, body)
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_top_with_streaming(self, mock):
        stream = [
            {
                "Processes": [
                    [
                        'jhonce',
                        '2417',
                        '2274',
                        '0',
                        'Mar01',
                        '?',
                        '00:00:01',
                        '/usr/bin/ssh-agent /bin/sh -c exec -l /bin/bash'
                        + '-c "/usr/bin/gnome-session"',
                    ],
                    ['jhonce', '5544', '3522', '0', 'Mar01', 'pts/1', '00:00:02', '-bash'],
                    ['jhonce', '6140', '3522', '0', 'Mar01', 'pts/2', '00:00:00', '-bash'],
                ],
                "Titles": ["UID", "PID", "PPID", "C", "STIME", "TTY", "TIME CMD"],
            }
        ]

        buffer = io.StringIO()
        for entry in stream:
            buffer.write(json.JSONEncoder().encode(entry))
            buffer.write("\n")

        adapter = mock.get(
            tests.LIBPOD_URL
            + "/containers/87e1325c82424e49a00abdd4de08009eb76c7de8d228426a9b8af9318ced5ecd/top"
            "?stream=True",
            text=buffer.getvalue(),
        )

        container = Container(attrs=FIRST_CONTAINER, client=self.client.api)
        top_stats = container.top(stream=True)

        self.assertIsInstance(top_stats, Iterable)
        for response, actual in zip(top_stats, stream):
            self.assertIsInstance(response, dict)
            self.assertDictEqual(response, actual)

        self.assertTrue(adapter.called_once)


if __name__ == '__main__':
    unittest.main()



import io
import requests
import json
import unittest

try:
    # Python >= 3.10
    from collections.abc import Iterable
except ImportError:
    # Python < 3.10
    from collections.abc import Iterable
from unittest.mock import patch

import requests_mock

from podman import PodmanClient, api, tests
from podman.domain.images import Image
from podman.errors import BuildError, DockerException

good_image_id = "032b8b2855fc"
good_stream = [
    {"stream": " ---\u003e a9eb17255234"},
    {"stream": "Step 1 : VOLUME /data"},
    {"stream": " ---\u003e Running in abdc1e6896c6"},
    {"stream": " ---\u003e 713bca62012e"},
    {"stream": "Removing intermediate container abdc1e6896c6"},
    {"stream": "Step 2 : CMD [\"/bin/sh\"]"},
    {"stream": " ---\u003e Running in dba30f2a1a7e"},
    {"stream": " ---\u003e 032b8b2855fc"},
    {"stream": "Removing intermediate container dba30f2a1a7e"},
    {"stream": f"{good_image_id}\n"},
]


class TestBuildCase(unittest.TestCase):
    """Test ImagesManager build().

    Note:
        Mock responses need to be coded for libpod returns.  The python bindings are responsible
            for mapping to compatible output.
    """

    def setUp(self) -> None:
        super().setUp()

        self.client = PodmanClient(base_url=tests.BASE_SOCK)

    def tearDown(self) -> None:
        super().tearDown()

        self.client.close()

    @patch.object(api, "create_tar")
    @patch.object(api, "prepare_containerfile")
    def test_build(self, mock_prepare_containerfile, mock_create_tar):
        mock_prepare_containerfile.return_value = "Containerfile"
        mock_create_tar.return_value = b"This is a mocked tarball."

        stream = good_stream
        buffer = io.StringIO()
        for entry in stream:
            buffer.write(json.JSONEncoder().encode(entry))
            buffer.write("\n")

        with requests_mock.Mocker() as mock:
            mock.post(
                tests.LIBPOD_URL + "/build"
                "?t=latest"
                "&buildargs=%7B%22BUILD_DATE%22%3A+%22January+1%2C+1970%22%7D"
                "&cpuperiod=10"
                "&extrahosts=%7B%22database%22%3A+%22127.0.0.1%22%7D"
                "&labels=%7B%22Unittest%22%3A+%22true%22%7D"
                "&manifest=example%3Av1.2.3"
                "&secrets=%5B%22id%3Dexample%2Csrc%3Dpodman-build-secret123%22%5D",
                text=buffer.getvalue(),
            )
            mock.get(
                tests.LIBPOD_URL + f"/images/{good_image_id}/json",
                json={
                    "Id": good_image_id,
                    "ParentId": "",
                    "RepoTags": ["fedora:latest", "fedora:33", "<none>:<none>"],
                    "RepoDigests": [
                        "fedora@sha256:9598a10fa72b402db876ccd4b3d240a4061c7d1e442745f1896ba37e1bf38664"
                    ],
                    "Created": 1614033320,
                    "Size": 23855104,
                    "VirtualSize": 23855104,
                    "SharedSize": 0,
                    "Labels": {},
                    "Containers": 2,
                },
            )

            image, logs = self.client.images.build(
                path="/tmp/context_dir",
                tag="latest",
                buildargs={
                    "BUILD_DATE": "January 1, 1970",
                },
                container_limits={
                    "cpuperiod": 10,
                },
                extra_hosts={"database": "127.0.0.1"},
                labels={"Unittest": "true"},
                manifest="example:v1.2.3",
                secrets=["id=example,src=podman-build-secret123"],
            )
            self.assertIsInstance(image, Image)
            self.assertEqual(image.id, good_image_id)
            self.assertIsInstance(logs, Iterable)

    @patch.object(api, "create_tar")
    @patch.object(api, "prepare_containerfile")
    def test_build_logged_error(self, mock_prepare_containerfile, mock_create_tar):
        mock_prepare_containerfile.return_value = "Containerfile"
        mock_create_tar.return_value = b"This is a mocked tarball."

        stream = [
            {"error": "We do not need any stinking badges."},
        ]

        buffer = io.StringIO()
        for entry in stream:
            buffer.write(json.JSONEncoder().encode(entry))
            buffer.write("\n")

        with requests_mock.Mocker() as mock:
            mock.post(
                tests.LIBPOD_URL + "/build",
                text=buffer.getvalue(),
            )

            with self.assertRaises(BuildError) as e:
                self.client.images.build(path="/tmp/context_dir")
            self.assertEqual(e.exception.msg, "We do not need any stinking badges.")

    @requests_mock.Mocker()
    def test_build_no_context(self, mock):
        mock.post(tests.LIBPOD_URL + "/build")
        with self.assertRaises(TypeError):
            self.client.images.build()

    @requests_mock.Mocker()
    def test_build_encoding(self, mock):
        mock.post(tests.LIBPOD_URL + "/build")
        with self.assertRaises(DockerException):
            self.client.images.build(path="/root", gzip=True, encoding="utf-8")

    @patch.object(api, "create_tar")
    @patch.object(api, "prepare_containerfile")
    def test_build_defaults(self, mock_prepare_containerfile, mock_create_tar):
        """Check the defaults used by images.build"""
        mock_prepare_containerfile.return_value = "Containerfile"
        mock_create_tar.return_value = b"This is a mocked tarball."

        stream = good_stream
        buffer = io.StringIO()
        for entry in stream:
            buffer.write(json.dumps(entry))
            buffer.write("\n")

        with requests_mock.Mocker() as mock:
            query = "?outputformat=" + (
                requests.utils.quote("application/vnd.oci.image.manifest.v1+json", safe='')
                + "&layers=True"
            )
            mock.post(
                tests.LIBPOD_URL + "/build" + query,
                text=buffer.getvalue(),
            )
            mock.get(
                tests.LIBPOD_URL + f"/images/{good_image_id}/json",
                json={
                    "Id": "unittest",
                },
            )
            img, _ = self.client.images.build(path="/tmp/context_dir")
        assert img.id == "unittest"


if __name__ == '__main__':
    unittest.main()



import unittest

import requests_mock

from podman import PodmanClient, tests
from podman.domain.manifests import Manifest, ManifestsManager

FIRST_MANIFEST = {
    "Id": "326dd9d7add24646a389e8eaa82125294027db2332e49c5828d96312c5d773ab",
    "names": "quay.io/fedora:latest",
}


class ManifestTestCase(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()

        self.client = PodmanClient(base_url=tests.BASE_SOCK)
        self.addCleanup(self.client.close)

    def test_podmanclient(self):
        manager = self.client.manifests
        self.assertIsInstance(manager, ManifestsManager)

    def test_list(self):
        with self.assertRaises(NotImplementedError):
            self.client.manifests.list()

    def test_name(self):
        manifest = Manifest()
        self.assertIsNone(manifest.name)

    @requests_mock.Mocker()
    def test_push(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL + "/manifests/quay.io%2Ffedora%3Alatest/registry/quay.io%2Ffedora%3Av1"
        )

        manifest = Manifest(attrs=FIRST_MANIFEST, client=self.client.api)
        manifest.push(destination="quay.io/fedora:v1")

        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_push_with_auth(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL
            + "/manifests/quay.io%2Ffedora%3Alatest/registry/quay.io%2Ffedora%3Av1",
            request_headers={
                "X-Registry-Auth": b"eyJ1c2VybmFtZSI6ICJ1c2VyIiwgInBhc3N3b3JkIjogInBhc3MifQ=="
            },
        )

        manifest = Manifest(attrs=FIRST_MANIFEST, client=self.client.api)
        manifest.push(
            destination="quay.io/fedora:v1", auth_config={"username": "user", "password": "pass"}
        )

        self.assertTrue(adapter.called_once)


if __name__ == '__main__':
    unittest.main()



import getpass
import unittest

import time

from podman import PodmanClient
from podman.tests.integration import base, utils


class AdapterIntegrationTest(base.IntegrationTest):
    def setUp(self):
        super().setUp()

    def test_ssh_ping(self):
        with PodmanClient(
            base_url=f"http+ssh://{getpass.getuser()}@localhost:22{self.socket_file}"
        ) as client:
            self.assertTrue(client.ping())

        with PodmanClient(
            base_url=f"ssh://{getpass.getuser()}@localhost:22{self.socket_file}"
        ) as client:
            self.assertTrue(client.ping())

    def test_unix_ping(self):
        with PodmanClient(base_url=f"unix://{self.socket_file}") as client:
            self.assertTrue(client.ping())

        with PodmanClient(base_url=f"http+unix://{self.socket_file}") as client:
            self.assertTrue(client.ping())

    def test_tcp_ping(self):
        podman = utils.PodmanLauncher(
            "tcp:localhost:8889",
            podman_path=base.IntegrationTest.podman,
            log_level=self.log_level,
        )
        try:
            podman.start(check_socket=False)
            time.sleep(0.5)

            with PodmanClient(base_url="tcp:localhost:8889") as client:
                self.assertTrue(client.ping())

            with PodmanClient(base_url="http://localhost:8889") as client:
                self.assertTrue(client.ping())
        finally:
            podman.stop()


if __name__ == '__main__':
    unittest.main()



import io
import json
import unittest
from types import GeneratorType

import requests_mock

from podman import PodmanClient, tests
from podman.domain.events import EventsManager


class EventsManagerTestCase(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()

        self.client = PodmanClient(base_url=tests.BASE_SOCK)

    def tearDown(self) -> None:
        super().tearDown()

        self.client.close()

    @requests_mock.Mocker()
    def test_list(self, mock):
        stream = [
            {
                "Type": "pod",
                "Action": "create",
                "Actor": {
                    "ID": "",
                    "Attributes": {
                        "image": "",
                        "name": "",
                        "containerExitCode": 0,
                    },
                },
                "Scope": "local",
                "Time": 1615845480,
                "TimeNano": 1615845480,
            }
        ]
        buffer = io.StringIO()
        for item in stream:
            buffer.write(json.JSONEncoder().encode(item))
            buffer.write("\n")

        adapter = mock.get(tests.LIBPOD_URL + "/events", text=buffer.getvalue())  # noqa: F841

        manager = EventsManager(client=self.client.api)
        actual = manager.list(decode=True)
        self.assertIsInstance(actual, GeneratorType)

        for item in actual:
            self.assertIsInstance(item, dict)
            self.assertEqual(item["Type"], "pod")

        actual = manager.list(decode=False)
        self.assertIsInstance(actual, GeneratorType)

        for item in actual:
            self.assertIsInstance(item, bytes)
            event = json.loads(item)
            self.assertEqual(event["Type"], "pod")


if __name__ == '__main__':
    unittest.main()



#   Copyright 2020 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""Integration Test Utils"""

import logging
import os
import shutil
import subprocess
import threading
from contextlib import suppress
from typing import Optional

import time

from podman.tests import errors

logger = logging.getLogger("podman.service")


class PodmanLauncher:
    """Podman service launcher"""

    def __init__(
        self,
        socket_uri: str,
        podman_path: Optional[str] = None,
        timeout: int = 0,
        privileged: bool = False,
        log_level: str = "WARNING",
    ) -> None:
        """create a launcher and build podman command"""
        podman_exe: str = podman_path
        if not podman_exe:
            podman_exe = shutil.which('podman')
        if podman_exe is None:
            raise errors.PodmanNotInstalled()

        self.socket_file: str = socket_uri.replace('unix://', '')
        self.log_level = log_level

        self.proc: Optional[subprocess.Popen[bytes]] = None
        self.reference_id = hash(time.monotonic())

        self.cmd: list[str] = []
        if privileged:
            self.cmd.append('sudo')

        self.cmd.append(podman_exe)

        logger.setLevel(logging.getLevelName(log_level))

        # Map from python to go logging levels, FYI trace level breaks cirrus logging
        self.cmd.append(f"--log-level={log_level.lower()}")

        if os.environ.get("container") == "oci":
            self.cmd.append("--storage-driver=vfs")

        self.cmd.extend(
            [
                "system",
                "service",
                f"--time={timeout}",
                socket_uri,
            ]
        )

        process = subprocess.run(
            [podman_exe, "--version"], check=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )
        self.version = str(process.stdout.decode("utf-8")).strip().split()[2]

    def start(self, check_socket=True) -> None:
        """start podman service"""
        logger.info(
            "Launching(%s) %s refid=%s",
            self.version,
            ' '.join(self.cmd),
            self.reference_id,
        )

        def consume_lines(pipe, consume_fn):
            with pipe:
                for line in iter(pipe.readline, b""):
                    consume_fn(line.decode("utf-8"))

        def consume(line: str):
            logger.debug(line.strip("\n") + f" refid={self.reference_id}")

        self.proc = subprocess.Popen(self.cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)  # pylint: disable=consider-using-with
        threading.Thread(target=consume_lines, args=[self.proc.stdout, consume]).start()

        if not check_socket:
            return

        # wait for socket to be created
        timeout = time.monotonic() + 30
        while not os.path.exists(self.socket_file):
            if time.monotonic() > timeout:
                raise subprocess.TimeoutExpired("podman service ", timeout)
            time.sleep(0.2)

    def stop(self) -> None:
        """stop podman service"""
        if not self.proc:
            return

        self.proc.terminate()
        try:
            return_code = self.proc.wait(timeout=15)
        except subprocess.TimeoutExpired:
            self.proc.kill()
            return_code = self.proc.wait()
        self.proc = None

        with suppress(FileNotFoundError):
            os.remove(self.socket_file)

        logger.info("Command return Code: %d refid=%s", return_code, self.reference_id)



#   Copyright 2020 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""system call integration tests"""

import podman.tests.integration.base as base
from podman import PodmanClient
from podman.errors import APIError


class SystemIntegrationTest(base.IntegrationTest):
    """system call integration test"""

    def setUp(self):
        super().setUp()
        self.client = PodmanClient(base_url=self.socket_uri)
        self.addCleanup(self.client.close)

    def test_info(self):
        """integration: system info call"""
        output = self.client.info()
        self.assertTrue('host' in output)

    def test_version(self):
        """integration: system version call"""
        output = self.client.version()
        self.assertTrue('Platform' in output)
        self.assertTrue('Version' in output)
        self.assertTrue('ApiVersion' in output)

    def test_show_disk_usage(self):
        """integration: system disk usage call"""
        output = self.client.df()
        self.assertTrue('Images' in output)
        self.assertTrue('Containers' in output)
        self.assertTrue('Volumes' in output)

    def test_login(self):
        """integration: system login call"""
        # here, we just test the sanity of the endpoint
        # confirming that we get through to podman, and get tcp rejected.
        with self.assertRaises(APIError) as e:
            next(
                self.client.login(
                    "fake_user", "fake_password", "fake_email@fake_domain.test", "fake_registry"
                )
            )
        self.assertRegex(
            e.exception.explanation,
            r"lookup fake_registry.+no such host",
        )

    def test_from_env(self):
        """integration: from_env() no error"""
        PodmanClient.from_env()



import unittest

import requests_mock

from podman import PodmanClient, tests
from podman.domain.pods import Pod
from podman.domain.pods_manager import PodsManager
from podman.errors import NotFound

FIRST_POD = {
    "ID": "c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8",
    "Name": "redis-ngnix",
}


class PodTestCase(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()

        self.client = PodmanClient(base_url=tests.BASE_SOCK)

    def tearDown(self) -> None:
        super().tearDown()

        self.client.close()

    def test_id(self):
        expected = {"Id": "c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8"}
        actual = Pod(attrs=expected)
        self.assertEqual(actual.id, expected["Id"])

        expected = {"Name": "redis-ngnix"}
        actual = Pod(attrs=expected)
        self.assertEqual(actual.name, expected["Name"])

    @requests_mock.Mocker()
    def test_kill(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL
            + "/pods/c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8/kill",
            json={
                "Errs": [],
                "Id": "c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8",
            },
        )

        pod = Pod(attrs=FIRST_POD, client=self.client.api)
        pod.kill()
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_kill_404(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL
            + "/pods/c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8/kill",
            status_code=404,
            json={
                "cause": "no such pod",
                "message": "no pod with name or ID xyz found: no such pod",
                "response": 404,
            },
        )

        pod = Pod(attrs=FIRST_POD, client=self.client.api)
        with self.assertRaises(NotFound):
            pod.kill()
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_pause(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL
            + "/pods/c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8/pause",
            json={
                "Errs": [],
                "Id": "c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8",
            },
        )

        pod = Pod(attrs=FIRST_POD, client=self.client.api)
        pod.pause()
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_pause_404(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL
            + "/pods/c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8/pause",
            status_code=404,
            json={
                "cause": "no such pod",
                "message": "no pod with name or ID xyz found: no such pod",
                "response": 404,
            },
        )

        pod = Pod(attrs=FIRST_POD, client=self.client.api)
        with self.assertRaises(NotFound):
            pod.pause()
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_remove(self, mock):
        adapter = mock.delete(
            tests.LIBPOD_URL
            + "/pods/c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8?force=True",
            json={
                "Errs": [],
                "Id": "c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8",
            },
        )
        pod_manager = PodsManager(client=self.client.api)
        pod = pod_manager.prepare_model(attrs=FIRST_POD)

        pod.remove(force=True)
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_restart(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL
            + "/pods/c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8/restart",
            json={
                "Errs": [],
                "Id": "c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8",
            },
        )

        pod = Pod(attrs=FIRST_POD, client=self.client.api)
        pod.restart()
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_start(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL
            + "/pods/c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8/start",
            json={
                "Errs": [],
                "Id": "c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8",
            },
        )

        pod = Pod(attrs=FIRST_POD, client=self.client.api)
        pod.start()
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_stop(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL
            + "/pods/c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8/stop?t=70",
            json={
                "Errs": [],
                "Id": "c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8",
            },
        )

        pod = Pod(attrs=FIRST_POD, client=self.client.api)
        pod.stop(timeout=70)
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_top(self, mock):
        body = {
            "Processes": [
                [
                    'jhonce',
                    '2417',
                    '2274',
                    '0',
                    'Mar01',
                    '?',
                    '00:00:01',
                    '/usr/bin/ssh-agent /bin/sh -c exec -l /bin/bash -c "/usr/bin/gnome-session"',
                ],
                ['jhonce', '5544', '3522', '0', 'Mar01', 'pts/1', '00:00:02', '-bash'],
                ['jhonce', '6140', '3522', '0', 'Mar01', 'pts/2', '00:00:00', '-bash'],
            ],
            "Titles": ["UID", "PID", "PPID", "C", "STIME", "TTY", "TIME CMD"],
        }
        adapter = mock.get(
            tests.LIBPOD_URL + "/pods"
            "/c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8/top"
            "?ps_args=aux&stream=False",
            json=body,
        )

        pod = Pod(attrs=FIRST_POD, client=self.client.api)
        actual = pod.top(ps_args="aux")
        self.assertDictEqual(actual, body)
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_unpause(self, mock):
        adapter = mock.post(
            tests.LIBPOD_URL
            + "/pods/c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8/unpause",
            json={
                "Errs": [],
                "Id": "c8b9f5b17dc1406194010c752fc6dcb330192032e27648db9b14060447ecf3b8",
            },
        )
        pod = Pod(attrs=FIRST_POD, client=self.client.api)
        pod.unpause()
        self.assertTrue(adapter.called_once)


if __name__ == '__main__':
    unittest.main()



import unittest

import requests_mock

from podman import PodmanClient, tests
from podman.domain.networks import Network
from podman.domain.networks_manager import NetworksManager

FIRST_NETWORK = {
    "Name": "podman",
    "Id": "2f259bab93aaaaa2542ba43ef33eb990d0999ee1b9924b557b7be53c0b7a1bb9",
    "Created": "2021-03-01T09:18:37.491308364-07:00",
    "Scope": "local",
    "Driver": "bridge",
    "EnableIPv6": False,
    "IPAM": {
        "Driver": "default",
        "Options": {},
        "Config": [{"Subnet": "10.88.0.0/16", "Gateway": "10.88.0.1"}],
    },
    "Internal": False,
    "Attachable": False,
    "Ingress": False,
    "ConfigFrom": {"Network": ""},
    "ConfigOnly": False,
    "Containers": {},
    "Options": {},
    "Labels": {},
}

SECOND_NETWORK = {
    "Name": "database",
    "Id": "3549b0028b75d981cdda2e573e9cb49dedc200185876df299f912b79f69dabd8",
    "Created": "2021-03-01T09:18:37.491308364-07:00",
    "Scope": "local",
    "Driver": "bridge",
    "EnableIPv6": False,
    "IPAM": {
        "Driver": "default",
        "Options": {},
        "Config": [{"Subnet": "10.88.0.0/16", "Gateway": "10.88.0.1"}],
    },
    "Internal": False,
    "Attachable": False,
    "Ingress": False,
    "ConfigFrom": {"Network": ""},
    "ConfigOnly": False,
    "Containers": {},
    "Options": {},
    "Labels": {},
}

FIRST_NETWORK_LIBPOD = {
    "name": "podman",
    "id": "2f259bab93aaaaa2542ba43ef33eb990d0999ee1b9924b557b7be53c0b7a1bb9",
    "driver": "bridge",
    "network_interface": "libpod_veth0",
    "created": "2022-01-28T09:18:37.491308364-07:00",
    "subnets": [
        {
            "subnet": "10.11.12.0/24",
            "gateway": "10.11.12.1",
            "lease_range": {
                "start_ip": "10.11.12.1",
                "end_ip": "10.11.12.63",
            },
        }
    ],
    "ipv6_enabled": False,
    "internal": False,
    "dns_enabled": False,
    "labels": {},
    "options": {},
    "ipam_options": {},
}

SECOND_NETWORK_LIBPOD = {
    "name": "database",
    "id": "3549b0028b75d981cdda2e573e9cb49dedc200185876df299f912b79f69dabd8",
    "created": "2021-03-01T09:18:37.491308364-07:00",
    "driver": "bridge",
    "network_interface": "libpod_veth1",
    "subnets": [
        {
            "subnet": "10.11.12.0/24",
            "gateway": "10.11.12.1",
            "lease_range": {
                "start_ip": "10.11.12.1",
                "end_ip": "10.11.12.63",
            },
        }
    ],
    "ipv6_enabled": False,
    "internal": False,
    "dns_enabled": False,
    "labels": {},
    "options": {},
    "ipam_options": {},
}


class NetworksManagerTestCase(unittest.TestCase):
    """Test NetworksManager area of concern.

    Note:
        Mock responses need to be coded for libpod returns.  The python bindings are responsible
            for mapping to compatible output.
    """

    def setUp(self) -> None:
        super().setUp()

        self.client = PodmanClient(base_url=tests.BASE_SOCK)
        self.addCleanup(self.client.close)

    def test_podmanclient(self):
        manager = self.client.networks
        self.assertIsInstance(manager, NetworksManager)

    @requests_mock.Mocker()
    def test_get(self, mock):
        mock.get(tests.LIBPOD_URL + "/networks/podman", json=FIRST_NETWORK)

        actual = self.client.networks.get("podman")
        self.assertIsInstance(actual, Network)
        self.assertEqual(
            actual.id, "2f259bab93aaaaa2542ba43ef33eb990d0999ee1b9924b557b7be53c0b7a1bb9"
        )

    @requests_mock.Mocker()
    def test_list_libpod(self, mock):
        mock.get(
            tests.LIBPOD_URL + "/networks/json",
            json=[FIRST_NETWORK_LIBPOD, SECOND_NETWORK_LIBPOD],
        )

        actual = self.client.networks.list()
        self.assertEqual(len(actual), 2)

        self.assertIsInstance(actual[0], Network)
        self.assertEqual(
            actual[0].id, "2f259bab93aaaaa2542ba43ef33eb990d0999ee1b9924b557b7be53c0b7a1bb9"
        )
        self.assertEqual(actual[0].attrs["name"], "podman")

        self.assertIsInstance(actual[1], Network)
        self.assertEqual(
            actual[1].id, "3549b0028b75d981cdda2e573e9cb49dedc200185876df299f912b79f69dabd8"
        )
        self.assertEqual(actual[1].name, "database")

    @requests_mock.Mocker()
    def test_create_libpod(self, mock):
        adapter = mock.post(tests.LIBPOD_URL + "/networks/create", json=FIRST_NETWORK_LIBPOD)

        network = self.client.networks.create("podman", dns_enabled=True, enable_ipv6=True)
        self.assertIsInstance(network, Network)

        self.assertEqual(adapter.call_count, 1)
        self.assertDictEqual(
            adapter.last_request.json(),
            {
                "name": "podman",
                "ipv6_enabled": True,
                "dns_enabled": True,
            },
        )

    @requests_mock.Mocker()
    def test_create_defaults(self, mock):
        adapter = mock.post(tests.LIBPOD_URL + "/networks/create", json=FIRST_NETWORK_LIBPOD)

        network = self.client.networks.create("podman")
        self.assertIsInstance(network, Network)

        self.assertEqual(adapter.call_count, 1)
        self.assertDictEqual(
            adapter.last_request.json(),
            {"name": "podman"},
        )

    @requests_mock.Mocker()
    def test_prune_libpod(self, mock):
        mock.post(
            tests.LIBPOD_URL + "/networks/prune",
            json=[
                {"Name": "podman", "Error": None},
                {"Name": "database", "Error": None},
            ],
        )

        actual = self.client.networks.prune()
        self.assertListEqual(actual["NetworksDeleted"], ["podman", "database"])


if __name__ == '__main__':
    unittest.main()



import io
import unittest

import requests_mock

from podman import PodmanClient, tests
from podman.domain.images import Image

FIRST_IMAGE = {
    "Id": "326dd9d7add24646a325e8eaa82125294027db2332e49c5828d96312c5d773ab",
    "ParentId": "",
    "RepoTags": ["fedora:latest", "fedora:33", "<none>:<none>"],
    "RepoDigests": [
        "fedora@sha256:9598a10fa72b402db876ccd4b3d240a4061c7d1e442745f1896ba37e1bf38664"
    ],
    "Created": 1614033320,
    "Size": 23855104,
    "VirtualSize": 23855104,
    "SharedSize": 0,
    "Labels": {},
    "Containers": 2,
}

SECOND_IMAGE = {
    "Id": "c4b16966ecd94ffa910eab4e630e24f259bf34a87e924cd4b1434f267b0e354e",
    "ParentId": "",
    "RepoDigests": [
        "fedora@sha256:4a877de302c6463cb624ddfe146ad850413724462ec24847832aa6eb1e957746"
    ],
    "Created": 1614033320,
    "Size": 23855104,
    "VirtualSize": 23855104,
    "SharedSize": 0,
    "Containers": 0,
}


class ImageTestCase(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()

        self.client = PodmanClient(base_url=tests.BASE_SOCK)

    def tearDown(self) -> None:
        super().tearDown()

        self.client.close()

    @requests_mock.Mocker()
    def test_history(self, mock):
        adapter = mock.get(
            tests.LIBPOD_URL
            + "/images/326dd9d7add24646a325e8eaa82125294027db2332e49c5828d96312c5d773ab/history",
            json=[
                {
                    "Id": "326dd9d7add24646a325e8eaa82125294027db2332e49c5828d96312c5d773ab",
                    "Comment": "",
                    "Created": 1614208404,
                    "CreatedBy": "2021-02-24T23:13:24+00:00",
                    "Tags": ["latest"],
                    "Size": 1024,
                }
            ],
        )
        image = Image(attrs=FIRST_IMAGE, client=self.client.api)

        history = image.history()
        self.assertEqual(history[0]["Id"], image.id)
        self.assertTrue(adapter.called_once)

    @requests_mock.Mocker()
    def test_reload(self, mock):
        update = FIRST_IMAGE.copy()
        update["Containers"] = 0

        adapter = mock.get(
            tests.LIBPOD_URL
            + "/images/326dd9d7add24646a325e8eaa82125294027db2332e49c5828d96312c5d773ab/json",
            [
                {"json": FIRST_IMAGE},
                {"json": update},
            ],
        )

        image = self.client.images.get(
            "326dd9d7add24646a325e8eaa82125294027db2332e49c5828d96312c5d773ab"
        )
        self.assertEqual(image.attrs["Containers"], 2)

        image.reload()
        self.assertEqual(image.attrs["Containers"], 0)
        self.assertTrue(adapter.call_count, 2)

    @requests_mock.Mocker()
    def test_save(self, mock):
        tarball = b'Yet another weird tarball...'
        body = io.BytesIO(tarball)

        adapter = mock.get(
            tests.LIBPOD_URL
            + "/images/326dd9d7add24646a325e8eaa82125294027db2332e49c5828d96312c5d773ab/get",
            body=body,
        )
        image = Image(attrs=FIRST_IMAGE, client=self.client.api)

        with io.BytesIO() as fd:
            for chunk in image.save():
                fd.write(chunk)
            self.assertEqual(fd.getbuffer(), tarball)
        self.assertTrue(adapter.called_once)


if __name__ == '__main__':
    unittest.main()



#   Copyright 2020 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""Network integration tests."""

import random
import unittest
from contextlib import suppress

import podman.tests.integration.base as base
from podman import PodmanClient
from podman.domain.ipam import IPAMConfig, IPAMPool
from podman.errors import NotFound


class NetworksIntegrationTest(base.IntegrationTest):
    """networks call integration test"""

    pool = IPAMPool(subnet="10.11.13.0/24", iprange="10.11.13.0/26", gateway="10.11.13.1")

    ipam = IPAMConfig(pool_configs=[pool])

    def setUp(self):
        super().setUp()

        self.client = PodmanClient(base_url=self.socket_uri)
        self.addCleanup(self.client.close)

    def tearDown(self):
        with suppress(NotFound):
            self.client.networks.get("integration_test").remove(force=True)

        super().tearDown()

    def test_network_crud(self):
        """integration: networks create and remove calls"""

        with self.subTest("Create Network"):
            network = self.client.networks.create(
                "integration_test",
                dns_enabled=False,
                ipam=NetworksIntegrationTest.ipam,
            )
            self.assertEqual(network.name, "integration_test")
            self.assertTrue(self.client.networks.exists(network.name))

        with self.subTest("Inspect Network"):
            network = self.client.networks.get("integration_test")
            self.assertEqual(network.name, "integration_test")

        with self.subTest("List Networks"):
            nets = self.client.networks.list()
            names = [i.name for i in nets]
            self.assertIn("integration_test", names)

        with self.subTest("Delete network"):
            network = self.client.networks.get("integration_test")
            network.remove(force=True)

            with self.assertRaises(NotFound):
                self.client.networks.get("integration_test")

    @unittest.skip("Skipping, libpod endpoint does not report container count")
    def test_network_connect(self):
        self.alpine_image = self.client.images.pull("quay.io/libpod/alpine", tag="latest")

        random_string = f"{random.getrandbits(160):x}"
        container = self.client.containers.create(
            self.alpine_image, command=["echo", random_string]
        )

        with self.subTest("Create Network"):
            network = self.client.networks.create(
                "integration_test",
                disabled_dns=True,
                enable_ipv6=False,
                ipam=NetworksIntegrationTest.ipam,
            )
            self.assertEqual(network.name, "integration_test")

        with self.subTest("Connect container to Network"):
            pre_count = network.containers
            network.connect(container=container)
            network.reload()

            post_count = network.containers
            self.assertLess(pre_count, post_count)

        with self.subTest("Disconnect container from Network"):
            pre_count = network.containers
            network.disconnect(container=container)
            network.reload()

            post_count = network.containers
            self.assertGreater(pre_count, post_count)

    @unittest.skip("Requires Podman 3.1")
    def test_network_prune(self):
        network = self.client.networks.create(
            "integration_test",
            disabled_dns=True,
            enable_ipv6=False,
            ipam=NetworksIntegrationTest.ipam,
        )
        self.assertEqual(network.name, "integration_test")

        report = self.client.networks.prune()
        self.assertIn(network.name, report["NetworksDeleted"])



import unittest

from podman.domain.pods_manager import PodsManager


class ManagerTestCase(unittest.TestCase):
    def test_prepare_model(self):
        with self.assertRaisesRegex(Exception, "^Can't create Pod from .*$"):
            PodsManager().prepare_model(attrs=("Sets", "Not", "supported"))


if __name__ == '__main__':
    unittest.main()



import pathlib
import csv
import re
import subprocess

try:
    from platform import freedesktop_os_release
except ImportError:

    def freedesktop_os_release() -> dict[str, str]:
        """This is a fallback for platforms that don't have the freedesktop_os_release function.
        Python < 3.10
        """
        path = pathlib.Path("/etc/os-release")
        with open(path) as f:
            reader = csv.reader(f, delimiter="=")
            return dict(reader)


def podman_version() -> tuple[int, ...]:
    cmd = ["podman", "info", "--format", "{{.Version.Version}}"]
    with subprocess.Popen(cmd, stdout=subprocess.PIPE) as proc:
        version = proc.stdout.read().decode("utf-8").strip()
    match = re.match(r"(\d+\.\d+\.\d+)", version)
    if not match:
        raise RuntimeError(f"Unable to detect podman version. Got \"{version}\"")
    version = match.group(1)
    return tuple(int(x) for x in version.split("."))


OS_RELEASE = freedesktop_os_release()
PODMAN_VERSION = podman_version()



import io
import random
import tarfile
import unittest

try:
    # Python >= 3.10
    from collections.abc import Iterator
except ImportError:
    # Python < 3.10
    from collections.abc import Iterator

import podman.tests.integration.base as base
from podman import PodmanClient
from podman.domain.containers import Container
from podman.domain.images import Image
from podman.errors import NotFound

# @unittest.skipIf(os.geteuid() != 0, 'Skipping, not running as root')


class ContainersIntegrationTest(base.IntegrationTest):
    """Containers Integration tests."""

    def setUp(self):
        super().setUp()

        self.client = PodmanClient(base_url=self.socket_uri)
        self.addCleanup(self.client.close)

        self.alpine_image = self.client.images.pull("quay.io/libpod/alpine", tag="latest")

        # TODO should this use podman binary instead?
        for container in self.client.containers.list():
            container.remove(force=True)

    def test_container_crud(self):
        """Test Container CRUD."""

        random_string = f"{random.getrandbits(160):x}"

        with self.subTest("Create from Alpine Image"):
            container = self.client.containers.create(
                self.alpine_image,
                command=["echo", random_string],
                ports={'2222/tcp': 3333, 2244: 3344},
            )
            self.assertIsInstance(container, Container)
            self.assertGreater(len(container.attrs), 0)
            self.assertIsNotNone(container.id)
            self.assertIsNotNone(container.name)
            self.assertIsInstance(container.image, Image)
            self.assertTrue(self.client.containers.exists(container.id))

            self.assertIn("quay.io/libpod/alpine:latest", container.image.tags)

        with self.subTest("Inspect Container"):
            actual = self.client.containers.get(container.id)
            self.assertIsInstance(actual, Container)
            self.assertEqual(actual.id, container.id)

            self.assertIn("2222/tcp", container.attrs["NetworkSettings"]["Ports"])
            self.assertEqual(
                "3333", container.attrs["NetworkSettings"]["Ports"]["2222/tcp"][0]["HostPort"]
            )
            self.assertIn("2244/tcp", container.attrs["NetworkSettings"]["Ports"])
            self.assertEqual(
                "3344", container.attrs["NetworkSettings"]["Ports"]["2244/tcp"][0]["HostPort"]
            )

        file_contents = b"This is an integration test for archive."
        file_buffer = io.BytesIO(file_contents)

        tar_buffer = io.BytesIO()
        with tarfile.open(fileobj=tar_buffer, mode="w") as tar:
            info = tarfile.TarInfo("/")
            info.type = tarfile.DIRTYPE
            tar.addfile(info)

            info = tarfile.TarInfo("unittest")
            info.size = len(file_buffer.getbuffer())
            tar.addfile(info, file_buffer)
            tarball = tar_buffer.getvalue()

        with self.subTest("Archive /root/unittest"):
            self.assertTrue(container.put_archive("/root", data=tarball))

            actual, _ = container.get_archive("/root")

            with io.BytesIO() as fd:
                for chunk in actual:
                    fd.write(chunk)
                fd.seek(0, 0)

                with tarfile.open(fileobj=fd, mode="r") as tar:
                    contents = tar.extractfile("root/unittest").read()
                self.assertEqual(contents, file_contents)

        with self.subTest("List containers --all"):
            containers = self.client.containers.list(all=True)
            self.assertGreater(len(containers), 0)
            ids = [i.id for i in containers]
            self.assertIn(container.id, ids)

        with self.subTest("Get container's logs"):
            container.start()
            container.wait(condition="exited")

            logs_iter = container.logs(stream=False)
            self.assertIsInstance(logs_iter, Iterator)

            logs = list(logs_iter)
            self.assertIn((random_string + "\n").encode("utf-8"), logs)

        with self.subTest("Delete Container"):
            container.remove()
            with self.assertRaises(NotFound):
                self.client.containers.get(container.id)

        with self.subTest("Run Container"):
            top_ctnr = self.client.containers.run(
                self.alpine_image, "/usr/bin/top", name="TestRunPs", detach=True
            )
            self.assertEqual(top_ctnr.status, "running")

            top_ctnr.pause()
            top_ctnr.reload()
            self.assertEqual(top_ctnr.status, "paused")

            top_ctnr.unpause()
            top_ctnr.reload()
            self.assertEqual(top_ctnr.status, "running")

            report = top_ctnr.top()
            # See https://github.com/containers/podman/pull/9892 for the service
            #   side fix requires podman >= 3.2
            # processes = [i.strip() for i in report["Processes"][0]]
            self.assertIn("/usr/bin/top", report["Processes"][0][-1])

            top_ctnr.stop()
            top_ctnr.reload()
            self.assertIn(top_ctnr.status, ("exited", "stopped"))

        with self.subTest("Create-Init-Start Container"):
            top_ctnr = self.client.containers.create(
                self.alpine_image, ["/usr/bin/top"], name="TestInitPs", detach=True
            )
            self.assertEqual(top_ctnr.status, "created")

            top_ctnr.init()
            top_ctnr.reload()
            self.assertEqual(top_ctnr.status, "initialized")

            top_ctnr.start()
            top_ctnr.reload()
            self.assertEqual(top_ctnr.status, "running")

            top_ctnr.stop()
            top_ctnr.reload()
            self.assertIn(top_ctnr.status, ("exited", "stopped"))

        with self.subTest("Prune Containers"):
            report = self.client.containers.prune()
            self.assertIn(top_ctnr.id, report["ContainersDeleted"])

            # SpaceReclaimed is the size of the content created during the running of the container
            # TODO: This should probably check if the podman version is >= 4.6 (guess)
            self.assertGreater(report["SpaceReclaimed"], 0)

            with self.assertRaises(NotFound):
                self.client.containers.get(top_ctnr.id)

    def test_container_commit(self):
        """Commit new image."""
        busybox = self.client.images.pull("quay.io/libpod/busybox", tag="latest")
        container = self.client.containers.create(
            busybox, command=["echo", f"{random.getrandbits(160):x}"]
        )

        image = container.commit(repository="busybox.local", tag="unittest")
        self.assertIn("localhost/busybox.local:unittest", image.attrs["RepoTags"])
        busybox.remove(force=True)

    def test_container_rm_anonymous_volume(self):
        with self.subTest("Check anonymous volume is removed"):
            container_file = io.StringIO("\n".join(["FROM alpine", "VOLUME myvol", "ENV foo=bar"]))
            test_img, _ = self.client.images.build(fileobj=container_file, tag="test-img", path=".")

            # get existing number of containers and volumes
            existing_containers = self.client.containers.list(all=True)
            existing_volumes = self.client.volumes.list()

            container = self.client.containers.create("test-img")
            container_list = self.client.containers.list(all=True)
            self.assertEqual(len(container_list), len(existing_containers) + 1)
            volume_list = self.client.volumes.list()
            self.assertEqual(len(volume_list), len(existing_volumes) + 1)

            # remove the container with v=True
            container.remove(v=True)
            container_list = self.client.containers.list(all=True)
            self.assertEqual(len(container_list), len(existing_containers))
            volume_list = self.client.volumes.list()
            self.assertEqual(len(volume_list), len(existing_volumes))
            # clean up
            self.client.images.remove(test_img)

    def test_container_labels(self):
        labels = {'label1': 'value1', 'label2': 'value2'}
        labeled_container = self.client.containers.create(self.alpine_image, labels=labels)
        unlabeled_container = self.client.containers.create(
            self.alpine_image,
        )

        # inspect and list have 2 different schemas so we need to verify that we can
        # successfully retrieve the labels on both
        try:
            # inspect schema
            self.assertEqual(labeled_container.labels, labels)
            self.assertEqual(unlabeled_container.labels, {})

            # list schema
            for container in self.client.containers.list(all=True):
                if container.id == labeled_container.id:
                    self.assertEqual(container.labels, labels)
                elif container.id == unlabeled_container.id:
                    self.assertEqual(container.labels, {})

        finally:
            labeled_container.remove(v=True)
            unlabeled_container.remove(v=True)

    def test_container_update(self):
        """Update container"""
        to_update_container = self.client.containers.run(
            self.alpine_image, name="to_update_container", detach=True
        )
        with self.subTest("Test container update changing the healthcheck"):
            to_update_container.update(health_cmd="ls")
            self.assertEqual(
                to_update_container.inspect()['Config']['Healthcheck']['Test'], ['CMD-SHELL', 'ls']
            )

        with self.subTest("Test container update disabling the healthcheck"):
            to_update_container.update(no_healthcheck=True)
            self.assertEqual(
                to_update_container.inspect()['Config']['Healthcheck']['Test'], ['NONE']
            )
        with self.subTest("Test container update passing payload and data"):
            to_update_container.update(
                restart_policy="always", health_cmd="echo", health_timeout="10s"
            )
            self.assertEqual(
                to_update_container.inspect()['Config']['Healthcheck']['Test'],
                ['CMD-SHELL', 'echo'],
            )
            self.assertEqual(
                to_update_container.inspect()['Config']['Healthcheck']['Timeout'], 10000000000
            )
            self.assertEqual(
                to_update_container.inspect()['HostConfig']['RestartPolicy']['Name'], 'always'
            )
        to_update_container.remove(v=True)


if __name__ == '__main__':
    unittest.main()
